#include <types.h>
#include <net/netdev.h>
#include <net/if_ether.h>
#include <string.h>
#include <console.h>

#include <h7210bd.h>

#define ETH_ALEN        6

#define min(a,b)	((a>b)?b:a)

#include "cs8900.h"
#include "udp.h"

EXTERN VOID eth_setup(struct netdev *nd);

//
// Structure to track receive packets
//
#define RCV_Q_SIZE 32
typedef struct _RCV_ENTRY {
    BYTE re_state;
    BYTE re_next;       // next card receive buffer
    UINT16 re_len;        // packet length
    UINT16 re_ptr;        // location in card's buffer
} RCV_ENTRY, *PRCV_ENTRY;

//
// re_state values
//
#define RCV_FREE 0
#define RCV_USED 1

STATIC RCV_ENTRY g_rcv_list[RCV_Q_SIZE];
STATIC UINT32 g_rcv_index;   // Next RCV_ENTRY to use
STATIC UINT32 g_rcv_next;    // Next RCV_ENTRY to indicate
STATIC UINT32 g_rcv_cnt;     // Number of packets to indicate

STATIC UINT32 transmitting = 0;

EXTERN struct netdev net_dev_cs8900;

STATIC VOLATILE UINT32 pbEthernetBase;




typedef struct 
{
    UINT16  configheader;
    UINT16  groupheader;
    UINT16  io_base;		/* I/O Base Address			*/
    UINT16  irq;			/* Interrupt Number			*/
    UINT16  groupheader2;
    UINT16  dma;			/* DMA Channel Numbers		*/
    UINT32  mem_base;		/* Memory Base Address		*/
    UINT32  rom_base;		/* Boot PROM Base Address	*/
    UINT32  rom_mask;		/* Boot PROM Address Mask	*/
    UINT16  groupheader3;
    UINT8   mac[6];			/* Individual Address		*/
    UINT16  cksum;
} CS8900_EEPROM;


STATIC INLINE VOID
CS8900_WRITE_PORT_USHORT(VOLATILE PUSHORT addr, USHORT val)
{
    *addr = val;
}

STATIC INLINE USHORT
CS8900_READ_PORT_USHORT(VOLATILE PUSHORT addr)
{
    return *addr;
}

#define WriteWord(Offset, Value)    CS8900_WRITE_PORT_USHORT((VOLATILE PUSHORT)(pbEthernetBase+Offset), (USHORT)(Value))
#define ReadWord(Offset)            CS8900_READ_PORT_USHORT((VOLATILE PUSHORT)(pbEthernetBase+Offset))


STATIC VOID InitRcvQueue(VOID)
{
    UINT32 n;

    for (n = 0; n < RCV_Q_SIZE; n++) {
        g_rcv_list[n].re_state = RCV_FREE;       
    }
    g_rcv_index = 0;
    g_rcv_next = 0;
    g_rcv_cnt = 0;
}


/*
 * CS8900 Register READ/WRITE Functions
 *
 */

STATIC INLINE VOID 
CS8900_WRITE(UINT32 Addr, UINT16 Data)
{
    WriteWord( PP_Address, Addr );
    WriteWord( PP_Data, Data );
}

STATIC INLINE UINT32
CS8900_READ(UINT32 Addr)
{
	UINT32  rVal;

    WriteWord( PP_Address, Addr );
    rVal = ReadWord( PP_Data );

	return rVal&0xffff;
}

STATIC INLINE VOID 
CS8900_WRITE32(UINT32 Addr, UINT32 Data)
{
	CS8900_WRITE(Addr, Data&0xffff);
	CS8900_WRITE(Addr + 2, (Data >> 16)& 0xffff);

    return;
}


STATIC INLINE UINT32 
CS8900_READ32(UINT32 Addr)
{
	UINT32 rVal;
	rVal = CS8900_READ(Addr) & 0xffff;
	rVal = rVal | ((CS8900_READ(Addr + 2) & 0x0ffff) << 16);

	return rVal;
}


STATIC INT32
HWWaiteepromReady( VOID )
{
	int TimeOut = 0x10000000;
	/* check to see if the EEPROM is ready, a timeout is used -
	   just in case EEPROM is ready when SI_BUSY in the
	   PP_SelfST is clear */
	while(CS8900_READ(PP_SelfST) & SIBUSY)
		if (TimeOut-- < 0)
			return -1;
	return 0;
}

STATIC INT32
HWGeteepromData(int off, int len, PUINT16 buffer )
{
	int i;

	console_printf("EEPROM data from %x ",off);
	console_printf("for %x:\n",len);
	for (i = 0; i < len; i++) 
	{
		if (HWWaiteepromReady() < 0)
		    return -1;
		/* Now send the EEPROM read command and EEPROM location to read */
		CS8900_WRITE(PP_EEPROMCommand, (off + i) | EEReadRegister);

		if (HWWaiteepromReady() < 0) 
		    return -1;
		buffer[i] = CS8900_READ(PP_EEPROMData)&0xffff;
		console_printf("%x ", buffer[i]);
	}
	console_printf("\n");
    return 0;
}

STATIC INT32
HWGeteepromCksum(int off, int len, PUINT16 buffer)
{
	int i, cksum;

	cksum = 0;
	for (i = 0; i < len; i++)
	{
		cksum += buffer[i]&0xFF;
		cksum += (buffer[i]>>8)&0xFF;
		console_printf(" buffer[i] = %x\r\n",buffer[i] );
		console_printf(" cksum = %x\r\n",cksum );
	}
	cksum &= 0xff;
	if (cksum == 0)
		return 0;
	return -1;
}

STATIC BOOL
HWReset( VOID )
{
    int TimeOut = 0x10000000;
    // reset
    CS8900_WRITE(PP_SelfCTL, CS8900_READ(PP_SelfCTL) | RESET );

    // wait 30ms 
   
    while( !(CS8900_READ(PP_SelfST) & INITD) )
		if (TimeOut-- < 0)
		{
            console_printf("\ncs89x0: RESET .... FAIL\n");
			return FALSE;
		}

    //console_printf("\ncs89x0: RESET .... SUCCESS\n");
    return TRUE;
}


//
//   HWInit - completely initialize the hardware and leave it in a
//          state that is ready to transmit and receive packets.
//
STATIC BOOL
HWInit(struct netdev *nd)
{
    UINT32  Val;
    CS8900_EEPROM   eeprom;


	/* verify EISA registration number for Cirrus Logic */
	if ((Val = CS8900_READ(PP_ProductID)) != EISA_REG_CODE) {
		console_printf("incorrect signature 0x%x\n",Val);
		return FALSE;
	}

    HWReset();

	/* verify chip version */
	Val = CS8900_READ(PP_ProductID + 2);
	if (CS_VERSION (Val) != CS8900A) {
		console_printf("unknown chip version 0x%x\n",CS_VERSION (Val));
		return FALSE;
	}
	console_printf( "CS8900A rev %c detected\n",'B' + REVISION (Val) - REV_B);

	/* setup interrupt number */
	CS8900_WRITE(PP_IntNum,0);

    //HWReset();

    console_printf("\nCS8900_READ(PP_SelfST) %x\n",CS8900_READ(PP_SelfST));
    
    ///* First check to see if an EEPROM is attached*/
    //if( (CS8900_READ(PP_SelfST) & EEPROM_PRESENT) == 0 )
    //    console_printf("\ncs89x0: No EEPROM, relying on command line....\n");
    //else 
    //if( HWGeteepromData(START_EEPROM_DATA,CHKSUM_LEN,eeprom_buff) < 0) {
    if( HWGeteepromData(0,0x11,(PUINT16)&eeprom) < 0) {
        console_printf("\ncs89x0: EEPROM read failed, relying on command line.\n");
    } else 
    //if( HWGeteepromCksum(START_EEPROM_DATA,CHKSUM_LEN,eeprom_buff) < 0) {
    if( HWGeteepromCksum(0,0x11,(PUINT16)&eeprom) < 0) {
        console_printf("\ncs89x0: EEPROM checksum bad, relying on command line\n");
    } 
    else 
    {
        INT     i;
        ///* get transmission control word  but keep the autonegotiation bits */
        //if (!lp->auto_neg_cnf) lp->auto_neg_cnf = eeprom_buff[AUTO_NEG_CNF_OFFSET/2];
        ///* Store adapter configuration */
        //if (!lp->adapter_cnf) lp->adapter_cnf = eeprom_buff[ADAPTER_CNF_OFFSET/2];
        ///* Store ISA configuration */
        //lp->isa_config = eeprom_buff[ISA_CNF_OFFSET/2];
        ///* store the initial memory base address */
        //dev->mem_start = eeprom_buff[PACKET_PAGE_OFFSET/2] << 8;

		for (i = 0; i < 6; i++)
		{
			nd->hw_addr[i] = eeprom.mac[i];
            console_printf("\nCS8900_MAC %x\n",eeprom.mac[i]);
        }

        /* receive only error free packets addressed to this card */
        CS8900_WRITE(PP_RxCTL, IAHashA | BroadcastA | RxOKA);

        /* do not generate any interrupts on receive operations */
        CS8900_WRITE(PP_RxCFG, 0);

        /* do not generate any interrupts on transmit operations */
        CS8900_WRITE(PP_TxCFG, 0);

        /* do not generate any interrupts on buffer operations */
        CS8900_WRITE(PP_BufCFG, 0);

        /* enable transmitter/receiver mode */
        CS8900_WRITE(PP_LineCTL, SerRxON | SerTxON);

        return TRUE;
    }


    return FALSE;
}


// This is called to initialze the ethernet low level driver.  The base address of the ethernet hardware
// is passed into the routine.  The routine will return TRUE for a successful initialization.
INT32
CS8900Open( struct netdev* nd)
{
    console_printf("+EDBG:CS8900Open\r\n");

    console_printf("EDBG:CS8900Open using I/O range at 0x%X\r\n", (UINT32)pbEthernetBase);
    InitRcvQueue();

    console_printf("-EDBG:CS8900Open\r\n");
    return TRUE;
}



// This is called to initialze the ethernet low level driver.  The base address of the ethernet hardware
// is passed into the routine.  The routine will return TRUE for a successful initialization.
INT32
CS8900Init( )
{
    INT         i;
//	PUCHAR pAttr;
//	PUCHAR pLastAttr;
//	INT32 Offset;      
//	UCHAR uByte;

	BOOL bRet;
	struct netdev *nd = &net_dev_cs8900;

	eth_setup(nd);

//	pAttr = (PUCHAR) CRYSTALLAN_ATTR_BASE_PHYS;
//
//    console_printf(" 0x%x,\r\n",pAttr);
//	console_printf(" 0x%x,\r\n",(((UINT32)pAttr)>>16));
//
//
//	for (pLastAttr = pAttr + 0x200; pAttr < pLastAttr; pAttr += 16) 
//	{
//		console_printf("0x%x:  %x %x %x %x ", pAttr,*pAttr,*(pAttr+2),*(pAttr+4),*(pAttr+6));
//		console_printf("%x %x %x %x\r\n", *(pAttr+8), *(pAttr+10), *(pAttr+12), *(pAttr+14));
//	}
//
//	console_printf("\nI/O read\n");
//
//		{
//#define CIS_R0 0x3F8
//			uByte = 0x20;			//16 bit I/O
//			Offset = CIS_R0;
//			pAttr = (PUCHAR) CRYSTALLAN_ATTR_BASE_PHYS;
//			pAttr += Offset; 
//			*pAttr = uByte;
//		} 

	
	nd->io_base = CS8900_IO_BASE_PHYS + 0x300;
	pbEthernetBase = (VOLATILE UINT32)nd->io_base;	// for macros
	bRet = HWInit(nd);

	console_printf("MAC addr = ");
    for( i = 0; i < 6; i++)
	    console_printf("%x.",nd->hw_addr[i]);

    console_printf("configure MAC address\n");
	for (i = 0; i < 6; i += 2)
        CS8900_WRITE( PP_IA+i, nd->hw_addr[i] | (nd->hw_addr[i+1]<<8) );


	console_printf("\n");

	return bRet;

}   // CS8900Init



//
// This routine should be called with a pointer to the ethernet frame data.  It is the caller's
//  responsibility to fill in all information including the destination and source addresses and
//  the frame type.  The length parameter gives the number of bytes in the ethernet frame.
// The routine will not return until the frame has been transmitted or an error has occured.  If
//  the frame transmits successfully, 0 is returned.  If an error occured, a message is sent to
//  the serial port and the routine returns non-zero.
//
INT32
CS8900SendFrame( struct netdev *nd, VOID *pbData, INT32 dwLength )
{
    volatile PUINT16    pAddr;
	INT32               TimeOut;
	UINT16              Tmp;


//console_printf("CS8900SendFrame 0...%x\n", pbData);
//console_printf("CS8900SendFrame 0...%d\n", dwLength);


RETRY:

    TimeOut = 0x100;
    /* initiate a transmit sequence */
    CS8900_WRITE( PP_TxCMD, TxStart(AfterAll));
    CS8900_WRITE( PP_TxLength, dwLength );

    /* Test to see if the chip has allocated memory for the packet */
	if( (CS8900_READ(PP_BusST) & Rdy4TxNOW) == 0) 
	{
        //while ( TimeOut-- < 0)
        /* Oops... this should not happen! */
        console_printf("cs: unable to send packet; retrying...\n");

        HWReset();
        goto RETRY;
    }

//console_printf("CS8900SendFrame 1...\n");

	/* Write the contents of the packet */
	/* assume even number of bytes */
    dwLength = (dwLength+1)>>1;
    pAddr = (PUINT16)pbData;
	for( ; dwLength > 0; dwLength--)
	{
//console_printf("CS8900SendFrame 0...%x\n", *pAddr);
        WriteWord(PP_RTDATA, *pAddr++);
    }

//console_printf("CS8900SendFrame 2...\n");
	/* wait for transfer to succeed */
    TimeOut = 0x10000000;
    while( (Tmp=CS8900_READ(PP_TxEvent)&~0x1F) == 0)
        if( TimeOut-- < 0)
        {
//console_printf("CS8900SendFrame Tx timeout...\n");
            break;
        }

//console_printf("CS8900SendFrame 3...\n");

    if( (Tmp & (Loss_of_CRS|TxOK)) != TxOK )
        console_printf("\ntransmission error %x\n", Tmp);

//console_printf("CS8900SendFrame 4...\n");
	return 0;

} // CS8900SendFrame()

#define PKTSIZE         1518
#define PKTSIZE_ALIGN   1536 
#define PKTALIGN        32
 //
// This routine is polled to find out if a frame has been received.  If there are no frames
//  in the RX FIFO, the routine will return 0.  If there was a frame that was received correctly,
//  it will be stored in pwData, otherwise it will be discarded.  
//
INT32
CS8900GetFrame(struct netdev *nd, VOID *pbData, INT32 wLength )
{
    INT32       i;
    UINT32      RxLen;
    PUINT16     pData;
    UINT32      Status;

    //console_printf("CS8900GetFrame...\n");

RETRY:
	Status = CS8900_READ(PP_RxEvent);
    if( (Status & RxOK) == 0 )
        goto RETRY;    

    pData = (PUINT16)pbData;

    Status = ReadWord(PP_RTDATA);    /* status : PP_RxStatus */
    RxLen = ReadWord(PP_RTDATA);     /* length : PP_RxLength */

//    console_printf("Status %x!\n", Status);
//    console_printf("RxLen  %d!\n", RxLen);

    if( RxLen > PKTSIZE_ALIGN + PKTALIGN )
        console_printf("packet too big %d!\n", RxLen);

    for( i = (RxLen+1)>>1 ; i > 0; i-- )
        *pData++ = ReadWord(PP_RTDATA);

//    console_printf("CS8900GetFrame-- ... %d\n", RxLen);
	return RxLen;
}

EXTERN const struct trans_ops eth_trans_ops;

STATIC const struct netdev_ops cs8900_ops = {
	CS8900Init,
	CS8900Open,
	CS8900SendFrame,
	CS8900GetFrame,
	"cs8900 lan",
};

struct netdev net_dev_cs8900 = {
	&cs8900_ops,
	&eth_trans_ops,
	"cs8900 lan",
};


