#include <types.h>
#include <net/netdev.h>
#include <net/if_ether.h>
#include <string.h>
#include <console.h>

#include <h7202bd.h>



#define min(a,b)	((a>b)?b:a)

#include "ne2000.h"
#include "udp.h"

EXTERN VOID eth_setup(struct netdev *nd);

//
// Structure to track receive packets
//
#define RCV_Q_SIZE 32
typedef struct _RCV_ENTRY {
    BYTE re_state;
    BYTE re_next;       // next card receive buffer
    UINT16 re_len;        // packet length
    UINT16 re_ptr;        // location in card's buffer
} RCV_ENTRY, *PRCV_ENTRY;

//
// re_state values
//
#define RCV_FREE 0
#define RCV_USED 1

STATIC RCV_ENTRY g_rcv_list[RCV_Q_SIZE];
STATIC UINT32 g_rcv_index;   // Next RCV_ENTRY to use
STATIC UINT32 g_rcv_next;    // Next RCV_ENTRY to indicate
STATIC UINT32 g_rcv_cnt;     // Number of packets to indicate

//
// Variables for remembering 8390 chip state
//
STATIC BYTE command_reg;
STATIC BYTE int_status;
STATIC BYTE NextPage;
STATIC BYTE overflowrestartflag;
STATIC BYTE pstart;
STATIC BYTE pstop;
STATIC BYTE transmitting;
STATIC UINT32 srambase;
STATIC UINT32 sramsize;

STATIC BYTE rcv_header[NIC_HEADER_LEN+16]; // NIC puts the current value of the Receive
#define rcv_status      rcv_header[0]     // Status Register in the frame header.
#define rcv_next_buff   rcv_header[1]   // Page number where next packet is/will be.
#define rcv_frame_len0  rcv_header[2]
#define rcv_frame_len1  rcv_header[3]   // This can be ignored since in high speed systems
                                        // it will be incorrect and can be calculated
                                        // using the page variables.
#define rcv_dest_addr0  rcv_header[4]
#define rcv_dest_addr1  rcv_header[5]

STATIC BYTE new_current_page;

STATIC BYTE tbb_start;
STATIC UINT16 curr_xmit_len;

STATIC BYTE erase_header[20];

// Mask to use for interrupts
STATIC BYTE bIntMask;
STATIC BYTE VOLATILE *pbEthernetBase;
//STATIC BYTE ethernetaddr[6];

// Bitmask for configuration options passed to us from bootloader/kernel.
STATIC UINT32 dwConfigOptions;

STATIC VOID InitRcvQueue(VOID);

EXTERN struct netdev net_dev_ne2000;

STATIC VOID
NE2000_WRITE_PORT_UCHAR(VOLATILE PUCHAR addr, UCHAR val)
{
    *addr = val;
}

STATIC UCHAR
NE2000_READ_PORT_UCHAR(VOLATILE PUCHAR addr)
{
    return *addr;
}

STATIC VOID
NE2000_WRITE_PORT_USHORT(VOLATILE PUSHORT addr, USHORT val)
{
    *addr = val;
}

STATIC USHORT
NE2000_READ_PORT_USHORT(VOLATILE PUSHORT addr)
{
    return *addr;
}

#define WriteByte(Offset, Value)    NE2000_WRITE_PORT_UCHAR((VOLATILE PUCHAR)(pbEthernetBase+Offset), Value)
#define ReadByte(Offset)            NE2000_READ_PORT_UCHAR((VOLATILE PUCHAR)(pbEthernetBase+Offset))
#define WriteWord(Offset, Value)    NE2000_WRITE_PORT_USHORT((VOLATILE PUSHORT)(pbEthernetBase+Offset), Value)
#define ReadWord(Offset)            NE2000_READ_PORT_USHORT((VOLATILE PUSHORT)(pbEthernetBase+Offset))

#define WriteCmd(Cmd)   WriteByte(NIC_COMMAND, Cmd)
#define ReadCmd()       ReadByte(NIC_COMMAND)
#define UsePage0()      WriteCmd(NIC_Page0)
#define UsePage1()      WriteCmd(NIC_Page1)
#define UsePage1Stop()  WriteCmd(NIC_Page1Stop)

//#define NE2000_DUMP_FRAMES 1
#ifdef NE2000_DUMP_FRAMES
STATIC VOID DumpEtherFrame( BYTE *pFrame, UINT16 cwFrameLength );
STATIC VOID DumpNE2000Header(VOID);
#endif

STATIC VOID iodelay(INT32 delay);
STATIC VOID HWAccess(UINT16 addr, UINT16 count, PUINT16 mem, BYTE direction);
#define INSB(addr, count, mem) HWAccess(((UINT16)(addr)), ((UINT16)(count)), (UINT16 *)mem, NIC_RemoteDmaRd)
#define OUTSB(addr, count, mem) HWAccess(((UINT16)(addr)), ((UINT16)(count)), (UINT16 *)mem, NIC_RemoteDmaWr)



///////////////////////////////////////////////////////////////////////////
//
//   iodelay - Wait by polling the command register for "delay" iterations.
//
///////////////////////////////////////////////////////////////////////////
STATIC VOID
iodelay(INT32 delay)
{
    while (delay) {
        ReadCmd();
        delay--;
    }
}   // iodelay

///////////////////////////////////////////////////////////////////////////
//
//  insb/outsb - Read/Write bytes or words from/to adapter ram
//
//  In word mode, the caller must ensure that adapter ram access starts
//  on a word boundary.
//
///////////////////////////////////////////////////////////////////////////
STATIC VOID
HWAccess(
    UINT16 addr,
    UINT16 count, 
    PUINT16 mem, 
    BYTE direction
    )
{
    UINT16 c;
//    PBYTE pb = (PBYTE)mem;

    if (!count) {
        console_printf("EDBG:NE2000:HWAccess - skipping 0 byte access!!!\r\n");
        return;     // Skip 0 length reads
    }
    
    WriteByte(NIC_INTR_STATUS, NIC_ISR_DMA_COMPLETE);

    // Set count, address, remote DMA address and direction,
    UsePage0();
    WriteByte(NIC_RMT_ADDR_LSB, (UCHAR)addr);
    WriteByte(NIC_RMT_ADDR_MSB, (UCHAR)(addr>>8));
    WriteByte(NIC_RMT_COUNT_LSB, (UCHAR)count);
    WriteByte(NIC_RMT_COUNT_MSB, (UCHAR)(count>>8));
    WriteCmd(direction);

    if (addr & 1) {     // Never start word IO on an odd adapter address
        console_printf("EDBG:NE2000:HWAccess SKIPPING ODD ADAPTER ADDRESS ACCESS!!!\r\n");
        return;
    }

    c = count >> 1;    // word count.
    if (direction == NIC_RemoteDmaWr) {
//{
//	INT32 i;
//	for(i=0; i < count; i++)
//		console_printf("%x = %x\n",i,*((char *)mem+i));
//}

        while (c) {
            WriteWord(NIC_RACK_NIC, *mem);
            mem++;
            c--;
        }
        if (count & 1) {    // trailing odd data byte will be on even adapter address
            WriteByte(NIC_RACK_NIC, *((PBYTE)mem));
        }
    } else {
        while (c) {
            *mem = ReadWord(NIC_RACK_NIC);
            mem++;
            c--;
        }
        if (count & 1) {
            *((PBYTE)mem) = ReadByte(NIC_RACK_NIC);
        }

//{
//	INT32 i;
//	for(i=0; i < count; i++)
//		console_printf("%x = %x\n",i,*((char *)mem-count+i));
//}

    }

    c = 0;
    while ((++c) &&
      !(ReadByte(NIC_INTR_STATUS) & NIC_ISR_DMA_COMPLETE));
    return;
}   // HWAccess


/////////////////////////////////////////////////////////////////////////////
//
//  HWStartAdapter - initialize the data size (BYTE or UINT16), set up the
//             receive configuration, and    unmask the receive and
//             transmit interrupts at the adapter.
//
/////////////////////////////////////////////////////////////////////////////
STATIC VOID
HWStartAdapter(VOID)
{
    UsePage0();
    WriteByte(NIC_DATA_CONFIG, NIC_DCR_FIFO | NIC_DCR_NORMAL | NIC_DCR_WORD_XFER);
    WriteByte(NIC_XMIT_CONFIG, 0);  // Initialize transmit configuration for normal operation.
    WriteByte(NIC_RCV_CONFIG, NIC_RECEIVE_BROADCAST);  // accept broadcast and directed packets.
    WriteByte(NIC_INTR_STATUS, 0xff );                  // ack all interrupts
    WriteCmd(NIC_CMD_ABORT | NIC_CMD_START );           // Put NIC in START mode.
}   // HWStartAdapter


/////////////////////////////////////////////////////////////////////////////
//
//  HWStopAdapter - shut down the adapter and leave it in loopback mode
//              preparatory to a reset.
//
/////////////////////////////////////////////////////////////////////////////
STATIC VOID
HWStopAdapter(VOID)
{
    UINT16    closeTimeout;

    WriteCmd(NIC_CMD_PAGE0 | NIC_CMD_ABORT | NIC_CMD_STOP); // Stop NIC from receiving or transmitting.
    WriteByte(NIC_INTR_STATUS, 0 );                         // mask all adapter interrupts
    WriteByte(NIC_RMT_COUNT_LSB, 0 );                       // Clear the remote byte count registers.
    WriteByte(NIC_RMT_COUNT_MSB, 0 );

    // Poll the card ISR for the RST bit.
    closeTimeout = 0;
    while ((++closeTimeout) &&
      !(ReadByte(NIC_INTR_STATUS) & NIC_ISR_RESET_STATUS));
    WriteByte(NIC_XMIT_CONFIG, 2);                          // Place NIC in internal loopback mode.
    WriteCmd(NIC_CMD_PAGE0 | NIC_CMD_START );               // Issue a start command. The NIC is still in loopback mode.
}   // HWStopAdapter


STATIC VOID
HWReset(VOID)
{
    HWStopAdapter();
    WriteByte(NIC_PAGE_START, pstart);
    WriteByte(NIC_BOUNDARY, (BYTE)(pstart+1));
    WriteByte(NIC_PAGE_STOP, pstop);
    UsePage1Stop();
    WriteByte(NIC_CURRENT, (BYTE)(pstart+1));
    WriteCmd(NIC_AbortDmaStop); // back to page0
    NextPage = pstart+1;
    HWStartAdapter();
}

// checkRam and ram_test are used at bind time.

#define RAMCHECK_SIZE    4    // RAM test pattern
STATIC BYTE ramCheck[RAMCHECK_SIZE] = { 0xAA, 0x55, 0xCC, 0x66 };


//
//  HWCheckRam - test adapter RAM by writing a test pattern and reading it.
//
//  Inputs:
//    addr  - internal adapter RAM buffer address
//    count - number of bytes to move
//
STATIC BOOL
HWCheckRam(UINT32 addr, UINT32 count)
{
    UINT32 i,j;
    BYTE buff[RAMCHECK_SIZE];

    // write the test pattern in 4 byte chunks
    for (i = 0; i < count; i += RAMCHECK_SIZE) {
        OUTSB((addr+i), min(RAMCHECK_SIZE,count), ramCheck);
    }

    // read it back and compare against the checkRam pattern.
    for (i = 0; i < count; i += RAMCHECK_SIZE) {
        INSB((addr+i), min(RAMCHECK_SIZE,count), buff);
        for (j = 0; j < min(RAMCHECK_SIZE,count); j++) {
            if (buff[j] != ramCheck[j]) {
                return FALSE;
            }
        }
    }
    return TRUE;
}

//
//  HWRamTest - figure out how much adapter RAM there is.
//
STATIC BOOL
HWRamTest(VOID)
{
    UINT32 ramEnd;
    BOOL bRet;

#define MAX_RAM_BASE 0x10000

    // find some RAM starting at 1K thru 64K. if it's greater then 64K, then
    // assume it isn't there.
    ramEnd = 0;
    HWStopAdapter();
    for (srambase = 1024; srambase < MAX_RAM_BASE; srambase += 1024) {
        if (HWCheckRam(srambase, 4)) {
            // found the starting point, now find out how much RAM there is.
            // assume it will increase in 1K chunks.
            for (ramEnd = srambase; !(ramEnd & 0xffff0000); ramEnd += 1024) {
                // RAM ends at the first location that fails the RAM test.
                if (!HWCheckRam(ramEnd, 4)) {
                    pstop = (BYTE)(ramEnd>>8);
                    break;
                }
            }
            break;
        }
    }

    // check for failure
    if ((srambase > MAX_RAM_BASE) || (srambase == ramEnd) || (ramEnd == 0)) {
        console_printf("EDBG:NE2000:HWRamTest fail. srambase = 0x%X, ramEnd = 0x%X\r\n",
            srambase, ramEnd);
        bRet = FALSE;
        goto hwrt_done;
    }

    // if ramEnd is >= 64K, then back off one 256 byte chunk to avoid the 16
    // bit wrap around.
    if (ramEnd & 0xffff0000) {
        pstop = 0xff;
    }

    //
    // Allocate 1 packet for transmit
    //
    tbb_start = (BYTE)(srambase>>8);
    pstart = tbb_start+6;   // Allocate 6 256 byte buffers for transmit

    // compute sramsize
    sramsize = ramEnd - srambase;

    console_printf("EDBG:NE2000:HWRamTest: srambase: 0x%X, sramsize: 0x%X, pstart: 0x%X\r\n", srambase, sramsize, pstart);

    // check the full RAM bank
    if (!HWCheckRam(srambase,sramsize)) {
        console_printf("EDBG:NE2000:HWRamTest full RAM check failed.\r\n");
        bRet = FALSE;
        goto hwrt_done;
    }

    bRet = TRUE;

hwrt_done:
    HWStartAdapter();
    return bRet;
}   // HWRamTest


//
//  HWClearMCRegs - Clear the adapter's multicast address registers.
//
STATIC VOID
HWClearMCRegs(VOID)
{
    UINT32 j;
    UsePage1();
    j = NIC_MC_ADDR;

    // Now clear out all multicast address registers.
    while (j < NIC_MC_ADDR+8) {
        WriteByte(j, 0);
        j++;
    }
    UsePage0();
}  // HWClearMCRegs


//
//   HWInit - completely initialize the hardware and leave it in a
//          state that is ready to transmit and receive packets.
//
STATIC BOOL
HWInit(struct netdev *nd)
{
    INT32 n;

    WriteCmd(NIC_AbortDmaStop); // Stop the NIC
    
    // Initialize Data Configuration register for auto remove.
    // This gets reset later on.
    WriteByte(NIC_DATA_CONFIG, NIC_DCR_FIFO|NIC_DCR_AUTO_INIT);

    // Bug workaround for SRAM memory protection.
    WriteByte(NIC_XMIT_START, 0xA0);
    WriteByte(NIC_XMIT_CONFIG, 0);
    WriteByte(NIC_RCV_CONFIG, NIC_RECEIVE_MONITOR_MODE);

    WriteByte(NIC_PAGE_START, 4);
    WriteByte(NIC_BOUNDARY,   4);
    WriteByte(NIC_PAGE_STOP, 0xff);
    WriteByte(NIC_XMIT_COUNT_LSB, 0x3c);
    WriteByte(NIC_XMIT_COUNT_MSB, 0);
    WriteByte(NIC_INTR_STATUS, 0xff);
    UsePage1Stop();
    WriteByte(NIC_CURRENT, 4);
    UsePage0();

    // If command register reflects the last command assume it is OK
    if (ReadCmd() != NIC_Page0) {
        console_printf("EDBG:NE2000Init:HWInit failed to read cmd reg\r\n");
        goto hw_dead;
    }
    
    WriteByte(NIC_RMT_COUNT_LSB, 55); // initialization errata
    WriteCmd(NIC_RemoteDmaRd);

    HWStartAdapter();
    if (!HWRamTest()) { // see how much adapter RAM
        console_printf("EDBG:NE2000Init:HWInit HWRamTest failed\r\n");
        goto hw_dead;
    }

    WriteCmd(NIC_AbortDmaStop);

    // Read the ethernet address
    // The NE2000 address PROM only decodes even addresses, so you
    // have to read WORDs and discard the high byte.
    WriteByte(NIC_RMT_COUNT_LSB, 12);
    WriteByte(NIC_RMT_COUNT_MSB, 0);
    WriteByte(NIC_RMT_ADDR_LSB, 0);
    WriteByte(NIC_RMT_ADDR_MSB, 0);
    WriteCmd(NIC_RemoteDmaRd);
    for (n = 0; n < 6; n++) {
        nd->hw_addr[n] = (BYTE)ReadWord(NIC_RACK_NIC);
    }

//	nd->hw_addr[0] = 0;
//	nd->hw_addr[1] = 0x48;
//	nd->hw_addr[2] = 0x54;
//	nd->hw_addr[3] = 0x82;
//	nd->hw_addr[4] = 0x1E;
//	nd->hw_addr[5] = 0xD6;
	
    // copy node address to page1 physical address registers
    UsePage1Stop();
    for (n = 0; n < 6; n++) {
        WriteByte(NIC_PHYS_ADDR+n, nd->hw_addr[n]);
    }

    HWClearMCRegs();            // Init multicast registers to 0
    WriteCmd(NIC_AbortDmaStop); // back to page0

    // Initialize the page start, stop, and boundary registers based on the
    // results of ram_test().
    WriteByte(NIC_PAGE_START, pstart);
    WriteByte(NIC_BOUNDARY, (BYTE)(pstart+1));
    WriteByte(NIC_PAGE_STOP, pstop);
    UsePage1Stop();
    WriteByte(NIC_CURRENT, (BYTE)(pstart+1));
    WriteCmd(NIC_AbortDmaStop); // back to page0
    NextPage = pstart+1;
    tbb_start = (BYTE)(srambase >> 8);
    
    memset(erase_header, 0xff, sizeof(erase_header));
    return TRUE;

hw_dead:
    WriteCmd(NIC_AbortDmaStop); // stop NIC
    return FALSE;
} // HWinit


// This is called to initialze the ethernet low level driver.  The base address of the ethernet hardware
// is passed into the routine.  The routine will return TRUE for a successful initialization.
INT32
NE2000Init( )
{
	PUCHAR pAttr;
	PUCHAR pLastAttr;
	INT32 Offset;      
	UCHAR uByte;

	BOOL bRet;
	struct netdev *nd = &net_dev_ne2000;

///	VOLATILE struct lcdregs *pLCDReg;
///	VOLATILE struct ppcreg *pPPCReg;

///	pLCDReg = (VOLATILE struct lcdregs *)LCD_BASE_PHYSICAL;
///	pPPCReg = (VOLATILE struct ppcreg *)PPC_BASE_PHYSICAL;



#if 1	
{
    // reset the crystal lan 
//	*(unsigned long *)(SA1110DB_PCR_PHYSICAL) =  PCR_SERIAL_EN;
//	msWait(300);
//	*(unsigned long *)(SA1110DB_PCR_PHYSICAL) =  PCR_SERIAL_EN | PCR_CF_RESET;
//	msWait(300);
//	*(unsigned long *)(SA1110DB_PCR_PHYSICAL) =  PCR_SERIAL_EN;
}

///#if 0
///{
///	VOLATILE struct gpioreg *pGPIOReg;
///	pGPIOReg=(VOLATILE struct gpioreg *)GPIO_BASE_PHYSICAL;
///	
///	pGPIOReg->gpdr.gp03 = 1;	// RESET_LAN_OUT
///	pGPIOReg->gpcr.gp03 = 1;	// RESET_LAN_OFF
///	msWait(300);
///	pGPIOReg->gpsr.gp03 = 1;	// RESET_LAN_ON
///	msWait(1000);
///	pGPIOReg->gpcr.gp03 = 1;	// RESET_LAN_OFF
///	msWait(300);
///}
#else
	while( pPPCReg->ppfr.lcd == 0)		// Disable LCD Block
		pLCDReg->lccr0.len = 0;
	
	pPPCReg->ppdr.l_bias = 1;		// Set the direction OUTPUT;
	
	pPPCReg->ppsr.l_bias = 0;		// Set to low
	msWait(300);
	pPPCReg->ppsr.l_bias = 1;		// Set to high
	msWait(1000);
	pPPCReg->ppsr.l_bias = 0;		// Set to low
	msWait(300);
#endif


#if 0
	//Start AX88790 LAN Part
	*(UCHAR *)( CRYSTALLAN_ATTR_BASE_PHYS+0x3C0) = (1<<0)|(1<<6); // Start ASIX LAN and Level IRQ
#endif


	pAttr = (PUCHAR) CRYSTALLAN_ATTR_BASE_PHYS;

    console_printf(" 0x%x,\r\n",pAttr);
	console_printf(" 0x%x,\r\n",(((UINT32)pAttr)>>16));


	for (pLastAttr = pAttr + 0x200; pAttr < pLastAttr; pAttr += 16) 
	{
		console_printf("0x%x:  %x %x %x %x ", pAttr,*pAttr,*(pAttr+2),*(pAttr+4),*(pAttr+6));
		console_printf("%x %x %x %x\r\n", *(pAttr+8), *(pAttr+10), *(pAttr+12), *(pAttr+14));
	}

	console_printf("\nI/O read\n");

		{
#define CIS_R0 0x3F8
			uByte = 0x20;			//16 bit I/O
			Offset = CIS_R0;
			pAttr = (PUCHAR) CRYSTALLAN_ATTR_BASE_PHYS;
			pAttr += Offset; 
			*pAttr = uByte;
		} 



//{
//	INT32 i,j;
//	for(j = 0; j < 0x40;j++)
//	{
//		for(i = 0; i < 16;i++)
//			console_printf(" 0x%2.2x,",*(char*)(CRYSTALLAN_ATTR_BASE_PHYS+j*16+i));
//		console_printf("\n");
//	}
//	
//}
	
	nd->io_base = CRYSTALLAN_IO_BASE_PHYS + 0x300;
	pbEthernetBase = (VOLATILE BYTE *)nd->io_base;	// for macros
	bRet = HWInit(nd);
	nd->hw_addr_len = 6;

	eth_setup(nd);
	
	console_printf("MAC addr = %x.%x.%x.%x.%x.%x\n", nd->hw_addr[0],nd->hw_addr[1],nd->hw_addr[2],nd->hw_addr[3],
											nd->hw_addr[4],nd->hw_addr[5]);
	return bRet;

}   // NE2000Init


// This is called to initialze the ethernet low level driver.  The base address of the ethernet hardware
// is passed into the routine.  The routine will return TRUE for a successful initialization.
INT32
NE2000Open( struct netdev* nd)
{
//    console_printf("+EDBG:NE2000Init\r\n");

    console_printf("EDBG:NE2000Init using I/O range at 0x%X\r\n", (UINT32)pbEthernetBase);
    InitRcvQueue();
    //bRet = HWInit();

//    console_printf("-EDBG:NE2000Init\r\n");
    return TRUE;
}   // NE2000Open

// Interrupts left disabled at init, call this function to turn them on
VOID
NE2000EnableInts()
{
    //console_printf("NE2000EnableInts\n");
    bIntMask = IMR_RCV|IMR_XMIT|IMR_RCV_ERR|IMR_XMIT_ERR|IMR_OVERFLOW;
    UsePage0();
    WriteByte(NIC_INTR_MASK, bIntMask);
}

VOID
NE2000DisableInts()
{
    //console_printf("NE2000DisableInts\n");
    bIntMask = 0;
    UsePage0();
    WriteByte(NIC_INTR_MASK, bIntMask);
}


//
//  HandleBufferOverflow - Perform the first 7 steps of the 11 step process to
//    recover from receive buffer overflow.  Step 8 is to remove packets from the
//  buffer which HandleReceive does. A flag is set to tell HandleReceive to do
//  steps 9-11.
//
//  Returns TRUE if current transmit needs resend.
// 
STATIC BOOL
HandleBufferOverflow(VOID)
{
    BOOL bResend = FALSE;

#ifdef DEBUG_SINGLE_CHAR
    console_printf("V"); // overflow
#endif

#define STOP_LOOP_CNT 6000
                                        // 1. remember TXP bit (it's already in _command_reg)       
    WriteCmd(NIC_AbortDmaStop);         // 2. take the controller offline
    iodelay(STOP_LOOP_CNT);             // 3. Wait 1.6ms for NIC to stop    
    WriteByte(NIC_RMT_COUNT_LSB, 0);    // 4. clear the remote byte count
    WriteByte(NIC_RMT_COUNT_MSB, 0);

    // The transmit may have completed while waiting for the NIC to stop
    int_status |= ReadByte(NIC_INTR_STATUS);    // BUGBUG???
    if (command_reg & NIC_CMD_XMIT) {
        if (int_status & (NIC_ISR_PACKET_XMITD|NIC_ISR_XMIT_ERR)) {
            bResend = TRUE;
        }
    }

    WriteByte(NIC_INTR_STATUS, int_status); // ack ints   
    WriteByte(NIC_XMIT_CONFIG, NIC_TCR_LOOPBACK1);  // 6. place NIC in loopback mode.    
    UsePage0();                                     // 7. restart in loopback mode
    overflowrestartflag = 1;
    return bResend;
}   // HandleBufferOverflow


//
//   alignframe - sometimes the frame gets shifted right by a UINT16.
//         Fix up the frame header and return with BX=packet offset
//         Increment DI to track alignments done/undone.
//
//   input: called == 0 -> first time in, shift frame right
//          called == 1 -> second time in, shift frame left
//
//   side effects: packet header (at rcv_status) is shifted around.
//          first case:     123456 -> 561234
//          second case:    123456 -> 345612
//
STATIC VOID
alignframe(INT32 called)
{
    BYTE tmp0;
    BYTE tmp1;

    if (called) {
        // shift the receive header left by a UINT16
        tmp0 = rcv_dest_addr0;
        tmp1 = rcv_dest_addr1;
        rcv_dest_addr0 = rcv_frame_len0;
        rcv_dest_addr1 = rcv_frame_len1;
        rcv_frame_len0 = rcv_status;
        rcv_frame_len1 = rcv_next_buff;
        rcv_status     = tmp0;
        rcv_next_buff  = tmp1;
    } else {
        // first time in for this packet, shift it right.
        tmp0 = rcv_status;
        tmp1 = rcv_next_buff;
        rcv_status     = rcv_frame_len0; 
        rcv_next_buff  = rcv_frame_len1;
        rcv_frame_len0 = rcv_dest_addr0;
        rcv_frame_len1 = rcv_dest_addr1;
        rcv_dest_addr0 = tmp0;
        rcv_dest_addr1 = tmp1;
    }    
}   // alignframe


//
//   check_rcv_header - check status and length fields and possibly call alignframe.
//
//  Return 0, 2, or 0xffff for failure
//
STATIC UINT16
check_rcv_header(VOID)
{
    INT32 alignframecalls = 0;
    BYTE prev_len0;
    BYTE prev_len1;
    UINT16 len;

crh_restart:
    if (alignframecalls >= 2) {
        return 0xffff;
    }
    if (rcv_status & 0x5E) {
        // receive status bits look wrong
        //console_printf("check_rcv_header: rcv_status = %B\r\n", rcv_status);
        alignframe(alignframecalls++);
    }

    // If it's a multicast or broadcast frame and we haven't realigned the
    // header, then see if the first byte of the destination address has its
    // lsb set. If it doesn't then it's in error.      
    if (rcv_status & NIC_RCV_STAT_MC_BC) {
        if (!alignframecalls) {
            if (!(rcv_dest_addr0 & 1)) {
                //console_printf("check_rcv_header: rcv_dest_addr0 = %B\r\n", rcv_dest_addr0);
                alignframe(alignframecalls++);
                goto crh_restart;
            }
        }
    }

    prev_len0 = rcv_frame_len0;
    prev_len1 = rcv_frame_len1;
    if (rcv_frame_len0 == rcv_frame_len1) {
        // If the length bytes are the same, the low byte may have been copied into the high byte.
        // We will compute the packet length based on rcv_next_buff, NextPage and the low byte
        // Calculate high byte of pkt length:  (in high speed systems, the low
        // byte of the packet length gets copied into rcv_frame_len0 and rcv_frame_len1)
        rcv_frame_len1 = rcv_next_buff - NextPage - 1;
        if (rcv_frame_len1 & 0x80) {
             rcv_frame_len1 = (pstop - NextPage) + (rcv_next_buff - pstart) - 1;
        }
        if (rcv_frame_len0 > 0xFC) {
            rcv_frame_len1++;
            rcv_frame_len0 = 0;
        }
        len = (UINT16)(rcv_frame_len0 + (rcv_frame_len1<<8));
        //console_printf("check_rcv_header: packet len = %d\r\n", len);
    }

    len = (UINT16)(rcv_frame_len0 + (rcv_frame_len1<<8));
//    if ((len > MAX_FRAME_SIZE + NIC_HEADER_LEN) || (len < MIN_FRAME_SIZE + NIC_HEADER_LEN)) {
    if (len > MAX_FRAME_SIZE + NIC_HEADER_LEN) {
        rcv_frame_len0 = prev_len0;
        rcv_frame_len1 = prev_len1;
        //console_printf("check_rcv_header: packet len = %d.\r\n", len);
        alignframe(alignframecalls++);
        goto crh_restart;
    }

    if ((rcv_next_buff < pstart) || (rcv_next_buff > pstop)) {
        alignframe(alignframecalls++);
        goto crh_restart;
    }
    return (alignframecalls == 1) ? 2 : 0;
}   // check_rcv_header


STATIC VOID
HWSetBoundary(BYTE next)
{
    if (next < pstart) {
        //console_printf("HWSetBoundary: next = %B\r\n", next);
        next = pstart;
    }
    if (next > pstop) {
        //console_printf("HWSetBoundary: next = %B\r\n", next);
        next = pstart;
    }
    WriteByte(NIC_BOUNDARY, (BYTE)((next == pstart) ? pstop-1 : next-1));
}

STATIC VOID InitRcvQueue(VOID)
{
    UINT32 n;

    for (n = 0; n < RCV_Q_SIZE; n++) {
        g_rcv_list[n].re_state = RCV_FREE;       
    }
    g_rcv_index = 0;
    g_rcv_next = 0;
    g_rcv_cnt = 0;
}

STATIC BOOL
QueueRcv(
    UINT16 packetlen,
    UINT16 packetptr,
    BYTE nextbuff
    )
{
    PRCV_ENTRY prcv = &(g_rcv_list[g_rcv_index]);

    if (prcv->re_state != RCV_FREE) {
        //console_printf("QueueRcv: No free receives!\r\n");
        return FALSE;
    }

    prcv->re_state = RCV_USED;
    prcv->re_next = nextbuff;
    prcv->re_len = packetlen;
    prcv->re_ptr = packetptr;
    //console_printf("QueueRcv: Queued %d at 0x%x, len %d\r\n", g_rcv_index, packetptr, packetlen);
    g_rcv_cnt++;
    g_rcv_index++;
    if (g_rcv_index == RCV_Q_SIZE) {
        g_rcv_index = 0;
    }
    return TRUE;

} // QueueRcv


//
//   HandleReceive - Check that there is a packet in the receive buffer and
//            check its length and status.  Then queue the receive if
//            the packet is OK.
//
STATIC VOID HandleReceive(VOID)
{
    BYTE rsr;   // receive status register
    UINT16 packetlen;
    UINT16 packetptr;
    INT32 loopcnt;
    UINT16 adjust;

#define MAX_RCV_CNT 32

    loopcnt = MAX_RCV_CNT;

    //console_printf("+HandleReceive\r\n");

    // Loop looking for more receive packets
hr_more_packets:
    if (!loopcnt) {
        //console_printf("HandleReceive Skipped %d packets\r\n", MAX_RCV_CNT);
        goto hr_exit;
    }
    loopcnt--;

    // If current_page register is not equal to NextPage
    // then we have a packet in the ring buffer.
    UsePage1();
    new_current_page = ReadByte(NIC_CURRENT);
    UsePage0();

    if ((int_status & NIC_ISR_RCV_ERR) && (!overflowrestartflag)) {
        rsr = ReadByte(NIC_RCV_STATUS);
        if (!(rsr & RSR_PACKET_OK)) {
            //console_printf("HandleReceive NIC_ISR_RCV_ERR (rsr = %B)\r\n", rsr);
            INSB((NextPage<<8), sizeof(rcv_header), rcv_header);
#ifdef NE2000_DUMP_FRAMES
            DumpNE2000Header();
#endif            
            /*
            if (rsr & RSR_CRC_ERROR) {
                console_printf("CRC Error\r\n");
            }
            if (rsr & RSR_MULTICAST) {
                console_printf("MULTICAST\r\n");
            }
            if (rsr & RSR_DISABLED) {
                console_printf("Receiver is disabled\r\n");
            }
            if (rsr & RSR_DEFERRING) {
                console_printf("Receiver is deferring\r\n");
            }
            */
#ifdef NE2000_DUMP_FRAMES
            DumpEtherFrame(&(rcv_dest_addr0), 14);
#endif
            InitRcvQueue();
            HWSetBoundary(new_current_page);
            NextPage = new_current_page;
            goto hr_exit;
        }
    }
    if (new_current_page == NextPage) {
        //console_printf("HandleReceive No new packets\r\n");
        goto hr_exit;
    }

    // There is another packet in the receive buffer.
    // Look at the packet's header
    INSB((NextPage<<8), sizeof(rcv_header), rcv_header);
    if ((adjust = check_rcv_header()) == 0xffff) {
        ///console_printf("HandleReceive check_rcv_header() failed!\r\n");
        // Advance receive buffer pointer on packet errors 
        NextPage = new_current_page;
        goto hr_exit;
    }

    packetlen = (UINT16)(rcv_frame_len1<<8)+(UINT16)rcv_frame_len0 - NIC_HEADER_LEN;
    rcv_next_buff = NextPage + rcv_frame_len1 + 1;
    if (rcv_next_buff > pstop) {
        rcv_next_buff -= pstop - pstart;
    }

    //
    // Filter out non-ARP broadcast packets.
    //
    if (rcv_status & NIC_RCV_STAT_MC_BC) {
        if ((dwConfigOptions & OPT_BROADCAST_FILTERING) &&
            ((rcv_header[16] != 8) || (rcv_header[17] != 6))) {
            NextPage = rcv_next_buff;
            console_printf("HandleReceive Dropping non-ARP broadcast packet!\r\n");
#ifdef NE2000_DUMP_FRAMES
            DumpEtherFrame(&(rcv_dest_addr0), 14);
#endif
            goto hr_more_packets;
        }
    }

    packetptr = (NextPage << 8) + NIC_HEADER_LEN + adjust;
    if (QueueRcv(packetlen, packetptr, rcv_next_buff)) {
        NextPage = rcv_next_buff;
        goto hr_more_packets;
    }

hr_exit:    // Done looking for receive packets

    if (!g_rcv_cnt) {
        //
        // We have caught up with receive packets, advance the boundary ptr
        //
        HWSetBoundary(NextPage);
    }

    if (overflowrestartflag) {
        // Do steps 9 and 10 of overflow recovery        
        WriteByte(NIC_INTR_STATUS, NIC_ISR_OVERWRITE);  // 9. reset the OVW bit        
        WriteByte(NIC_XMIT_CONFIG, NIC_TCR_NORMAL);     // 10. Get out of loopback mode.
        overflowrestartflag = 0;    
    }
    //console_printf("-HandleReceive\r\n");
}   // HandleReceive



STATIC VOID HandleTransmit(VOID)
{
    // check for transmit errors
    if ((int_status & NIC_ISR_XMIT_ERR) || (!(ReadByte(NIC_XMIT_STATUS) & NIC_TSR_SUCCESS))) {
        iodelay(15);
        WriteByte(NIC_XMIT_CONFIG, NIC_TCR_LOOPBACK1);  // place NIC in loopback mode.
        UsePage0();                                     // start in loopback mode.
        ReadCmd();
        WriteByte(NIC_XMIT_CONFIG, NIC_TCR_NORMAL);     // get out of loopback mode.
        UsePage0();                                     // start the NIC
        ReadCmd();
    }
    transmitting = 0;
}

VOID NE2000_ISR(VOID)
{

#define ISR_LOOP_MAX 32

    INT32 loopcnt = ISR_LOOP_MAX;

    command_reg = ReadCmd();
    UsePage0();
    int_status = ReadByte(NIC_INTR_STATUS);

    while (int_status & NIC_ISR_MOREWORK) {
        WriteByte(NIC_INTR_STATUS, int_status); // ack the interrupts    
        //
        // Respond to the interrupt status register bits.
        //
        if (int_status & NIC_ISR_OVERWRITE) {
//console_printf("NE2000_ISR : Handle Overflow\n");
            HandleBufferOverflow();
        }
    
        //if (int_status & (NIC_ISR_PACKET_RCVD|NIC_ISR_RCV_ERR)) {
            HandleReceive();
        //}

        if (int_status & (NIC_ISR_PACKET_XMITD|NIC_ISR_XMIT_ERR)) {
//console_printf("NE2000_ISR : Handle Transmit\n");
            HandleTransmit();
        }

		if (!-- loopcnt)
			break;
			
	    command_reg = ReadCmd();
		int_status = ReadByte(NIC_INTR_STATUS);
    }
}   // NE2000_ISR


UINT32
NE2000GetPendingInts()
{
    //console_printf("+NE2000GetPendingInts\r\n");
    NE2000_ISR();
    //console_printf("-NE2000GetPendingInts\r\n");
    if (g_rcv_cnt) {
#ifdef DEBUG_SINGLE_CHAR
        console_printf("R"); // indicating a received packet
#endif
        return INTR_TYPE_RX;    
    }
#ifdef DEBUG_SINGLE_CHAR
    console_printf("B"); // discarding non-ARP broadcast traffic.
#endif
    return 0;    
}   // NE2000GetPendingInts


//
// This routine is polled to find out if a frame has been received.  If there are no frames
//  in the RX FIFO, the routine will return 0.  If there was a frame that was received correctly,
//  it will be stored in pwData, otherwise it will be discarded.  
//
INT32
NE2000GetFrame(struct netdev *nd, VOID *pbData, INT32 wLength )
{
    PRCV_ENTRY prcv;
    UINT16 len;

    //console_printf("+NE2000GetFrame\r\n");
    //EDBG_DEBUGLED(LED_GETFRAME_ENTRY,0);

    len = wLength;
    wLength = 0;
    if (!g_rcv_cnt) {
        NE2000_ISR();
    }
    if (g_rcv_cnt) 
    {
        prcv = &(g_rcv_list[g_rcv_next]);
        if (prcv->re_state == RCV_USED) 
        {
            //console_printf("NE2000GetFrame: reading %d (ptr 0x%x, len %d)\r\n", g_rcv_next, prcv->re_ptr, prcv->re_len);
            if (prcv->re_len < len) 
            {
                len = prcv->re_len;
            }
            INSB(prcv->re_ptr, len, pbData);
            HWSetBoundary(prcv->re_next);
            wLength = len;
            prcv->re_state = RCV_FREE;
            g_rcv_cnt--;
            g_rcv_next++;
            if (g_rcv_next == RCV_Q_SIZE) 
            {
                g_rcv_next = 0;
            }
    	}
    } 
    else 
    {
#ifdef DEBUG_SINGLE_CHAR
        console_printf("Z"); // Receive buffer empty
#endif
    }

    //console_printf("-NE2000GetFrame wLength = %d\r\n", wLength);
    //EDBG_DEBUGLED(LED_GETFRAME_EXIT,0);
    return wLength;
} // NE2000GetFrame()


//
// This routine should be called with a pointer to the ethernet frame data.  It is the caller's
//  responsibility to fill in all information including the destination and source addresses and
//  the frame type.  The length parameter gives the number of bytes in the ethernet frame.
// The routine will not return until the frame has been transmitted or an error has occured.  If
//  the frame transmits successfully, 0 is returned.  If an error occured, a message is sent to
//  the serial port and the routine returns non-zero.
//
INT32
//NE2000SendFrame( BYTE *pbData, UINT32 dwLength )
NE2000SendFrame( struct netdev *nd, VOID *pbData, INT32 dwLength )
{
    INT32 nIters;

    //console_printf("+NE2000SendFrame\r\n");
    //
    // Loop here until the request is satisfied, or we timeout
    //
    for (nIters = 0; transmitting; nIters ++) {
        iodelay(20);
        NE2000_ISR();
        if (nIters > 1000) {
            console_printf("NE2000SendFrame Timed out waiting for transmit buffers\n");
            //
            // 2 seconds ought to be enough to transmit a packet!!! Reset for transmit errors
            // 
            iodelay(15);
            WriteByte(NIC_XMIT_CONFIG, NIC_TCR_LOOPBACK1);  // place NIC in loopback mode.
            UsePage0();                                     // start in loopback mode.
            ReadCmd();
            WriteByte(NIC_XMIT_CONFIG, NIC_TCR_NORMAL);     // get out of loopback mode.
            UsePage0();                                     // start the NIC
            ReadCmd();

            HWReset();
            transmitting = 0;
        }
    }

    curr_xmit_len = (UINT16)dwLength;
    //console_printf("NE2000SendFrame %d byte packet\r\n", curr_xmit_len);
#ifdef DEBUG_SINGLE_CHAR
    console_printf("S"); // Send packet
#endif
    OUTSB(srambase, curr_xmit_len, pbData);      
    WriteByte(NIC_XMIT_START, tbb_start);
    if (curr_xmit_len < 60) {
        curr_xmit_len = 60;
    }
    if (curr_xmit_len > 1536) {
        curr_xmit_len = 1536;
    }
    WriteByte(NIC_XMIT_COUNT_LSB, (BYTE)curr_xmit_len);
    WriteByte(NIC_XMIT_COUNT_MSB, (BYTE)(curr_xmit_len>>8));
    transmitting = 1;
    WriteCmd(NIC_Transmit);
    iodelay(80);
    NE2000_ISR();
//    console_printf("-NE2000SendFrame\r\n");
    return 0;
} // NE2000SendFrame()


#ifdef NE2000_DUMP_FRAMES
STATIC VOID
DumpEtherFrame( BYTE *pFrame, UINT16 cwFrameLength )
{
    INT32 i,j;

    console_printf( "Frame Buffer Address: 0x%X\r\n", pFrame );
    console_printf( "To: %B:%B:%B:%B:%B:%B  From: %B:%B:%B:%B:%B:%B  Type: 0x%H  Length: %u\r\n",
        pFrame[0], pFrame[1], pFrame[2], pFrame[3], pFrame[4], pFrame[5],
        pFrame[6], pFrame[7], pFrame[8], pFrame[9], pFrame[10], pFrame[11],
        ntohs(*((UINT16 *)(pFrame + 12))), cwFrameLength );
    for( i = 0; i < cwFrameLength / 16; i++ ) {
        for( j = 0; j < 16; j++ )
            console_printf( " %B", pFrame[i*16 + j] );
        console_printf( "\r\n" );
    }
    for( j = 0; j < cwFrameLength % 16; j++ )
        console_printf( " %B", pFrame[i*16 + j] );
    console_printf( "\r\n" );
}

STATIC VOID
DumpNE2000Header(VOID)
{
    console_printf( "NE2000 Frame Header at %B: Status: %B, Next Buffer: %B, Len0: %B, Len1: %B\r\n",
        NextPage, rcv_status, rcv_next_buff, rcv_frame_len0, rcv_frame_len1);
}
#endif  // NE2000_DUMP_FRAMES

BOOL
NE2000ReadEEPROM( UINT16 EEPROMAddress , UINT16 *pwVal)
{
    return FALSE;
}

BOOL
NE2000WriteEEPROM( UINT16 EEPROMAddress, UINT16 Data )
{
    return FALSE;
}

//
// This routine is used by the OAL to configure the debug Ethernet driver. Currently
// the following options are defined:
//     OPT_BROADCAST_FILTERING  -- If set, filter out all broadcast packets except ARP packets
// 
UINT32
NE2000SetOptions(UINT32 dwOptions)
{
    UINT32 dwOldOptions = dwConfigOptions;
    dwConfigOptions = dwOptions;
    return dwOldOptions;
}


EXTERN const struct trans_ops eth_trans_ops;

STATIC const struct netdev_ops ne2000_ops = {
	NE2000Init,
	NE2000Open,
	NE2000SendFrame,
	NE2000GetFrame,
	"ne2000 Compatilble",
};

struct netdev net_dev_ne2000 = {
	&ne2000_ops,
	&eth_trans_ops,
	"ne2000 Compatilble",
};


