#ifndef __ATMEL_GPIO_H__
#define __ATMEL_GPIO_H__

#define writel(value, address) 	(*(volatile unsigned int *)(address)) = (value)
#define readl(address) 	        (*(volatile unsigned int *)(address))

// the following values are used to compute the LOGINTR associated with a PIO pin.
// For example LOGINTR for PIOA pin 4 is LOGINTR_BASE_PIOA + 4
#define LOGINTR_BASE_PIOA			64
#define LOGINTR_BASE_PIOB			96                           
#define LOGINTR_BASE_PIOC			128
#define LOGINTR_BASE_PIOD			160                        
#define LOGINTR_BASE_PIOE			192                        

//#define LOGINTR_OHCI				40
//#define LOGINTR_EHCI				41

/* these pin numbers double as IRQ numbers, like AT91C_ID_* values */
#define PIO_NB_IO		32 /* Number of IO handled by one PIO controller */
#define	AT91C_PIN_PA(io)	(0 * PIO_NB_IO + io)
#define	AT91C_PIN_PB(io)	(1 * PIO_NB_IO + io)
#define	AT91C_PIN_PC(io)	(2 * PIO_NB_IO + io)
#define	AT91C_PIN_PD(io)	(3 * PIO_NB_IO + io)
#define	AT91C_PIN_PE(io)	(4 * PIO_NB_IO + io)



#define SAM9_PIO_INTR_ARRAY	{\
		{AT91C_BASE_PIOA,32,0,0,LOGINTR_BASE_PIOA,SAMA5D3_ID_PIOA,0},\
		{AT91C_BASE_PIOB,32,0,0,LOGINTR_BASE_PIOB,SAMA5D3_ID_PIOB,0},\
		{AT91C_BASE_PIOC,32,0,0,LOGINTR_BASE_PIOC,SAMA5D3_ID_PIOC,0},\
		{AT91C_BASE_PIOD,32,0,0,LOGINTR_BASE_PIOD,SAMA5D3_ID_PIOD,0},\
		{AT91C_BASE_PIOE,32,0,0,LOGINTR_BASE_PIOE,SAMA5D3_ID_PIOE,0},\
		}


//------------------------------------------------------------------------------
//                                                               Defines & Types
//------------------------------------------------------------------------------
/* I/O type */
enum  pio_type 
{
	PIO_PERIPH_A,
	PIO_PERIPH_B,
	PIO_PERIPH_C,
	PIO_PERIPH_D,
	PIO_INPUT,
	PIO_OUTPUT_0,
	PIO_OUTPUT_1,
	PIO_UNDEFINED
};

enum  pio_irq_type 
{
	IRQ_TYPE_NONE,
	IRQ_TYPE_EDGE_RISING,
	IRQ_TYPE_EDGE_FALLING,
	IRQ_TYPE_LEVEL_LOW,
	IRQ_TYPE_LEVEL_HIGH,
	IRQ_TYPE_EDGE_BOTH,
};

/* I/O attributes */
#define PIO_DEFAULT   (0 << 0)
#define PIO_PULLUP    (1 << 0)
#define PIO_DEGLITCH  (1 << 1)
#define PIO_OPENDRAIN (1 << 2)
#define PIO_PULLDN    (1 << 3)

struct pio_desc 
{
	const char *pin_name;   /* Pin Name */
	unsigned int pin_num;   /* Pin number */
	unsigned int dft_value; /* Default value for outputs */
	unsigned char attribute;
	enum pio_type type;
};

typedef struct 
{
	AT91PS_PIO pBase;
	DWORD nbPin; //Not every PIO controller support the same number of pin.
	DWORD currentPin;//This is used to maintain a round robin priority system within the same PIO bank
	DWORD status; // this is use to maintain the value of PIO_ISR because it's cleared after each reading
	DWORD logintrBase; //This is the logintr associated with pin 0 of the bank
	DWORD deviceID; //this is to configure the AIC
	DWORD resumeMask;
} PIO_BANK_DESCRIPTION;

#define END_OF_PIO {NULL, 0, 0, 0, PIO_UNDEFINED}

//------------------------------------------------------------------------------
//                                                            Exported functions
//------------------------------------------------------------------------------

#ifdef __cplusplus
extern "C" int pio_setup (const struct pio_desc *pio_desc, int nb_pio);
extern "C" void pio_set_value (unsigned pin, int value);
extern "C" int pio_get_value (unsigned pin);
#else
/* pio_device_pio_setup: Configure PIO in periph mode according to the platform informations */
extern int pio_setup (const struct pio_desc *pio_desc, int nb_pio);
extern void pio_set_value (unsigned pin, int value);
extern int pio_get_value (unsigned pin);
#endif

//*----------------------------------------------------------------------------
//* \fn    AT91F_PIO_GetInput
//* \brief Return PIO input value
//*----------------------------------------------------------------------------
static __inline unsigned int AT91F_PIO_GetInput( // \return PIO input
	AT91PS_PIO pPio) // \arg  pointer to a PIO controller
{
	return pPio->PIO_PDSR;
}

//*----------------------------------------------------------------------------
//* \fn    AT91F_PIO_IsInputSet
//* \brief Test if PIO is input flag is active
//*----------------------------------------------------------------------------
static __inline int AT91F_PIO_IsInputSet(
	AT91PS_PIO pPio,   // \arg  pointer to a PIO controller
	unsigned int flag) // \arg  flag to be tested
{
	return (AT91F_PIO_GetInput(pPio) & flag);
}

//*----------------------------------------------------------------------------
//* \fn    AT91F_PIO_PullupActive
//* \brief Enable pullup on PIO
//*----------------------------------------------------------------------------
static __inline void AT91F_PIO_PullupActive(
	AT91PS_PIO pPio,             // \arg pointer to a PIO controller
	unsigned int pullupEnable)   // \arg enable pullup on PIO
{
		// Connect Pullup
	pPio->PIO_PPUER = pullupEnable;
}


//*----------------------------------------------------------------------------
//* \fn    AT91F_PIO_CfgOutput
//* \brief Enable PIO in output mode
//*----------------------------------------------------------------------------
static __inline void AT91F_PIO_CfgOutput(
	AT91PS_PIO pPio,             // \arg pointer to a PIO controller
	unsigned int pioEnable)      // \arg PIO to be enabled
{
	pPio->PIO_PER = pioEnable; // Set in PIO mode
	pPio->PIO_OER = pioEnable; // Configure in Output
}

//*----------------------------------------------------------------------------
//* \fn    AT91F_PIO_CfgInput
//* \brief Enable PIO in input mode
//*----------------------------------------------------------------------------
static __inline void AT91F_PIO_CfgInput(
	AT91PS_PIO pPio,             // \arg pointer to a PIO controller
	unsigned int inputEnable)      // \arg PIO to be enabled
{
	// Disable output
	pPio->PIO_ODR  = inputEnable;
	pPio->PIO_PER  = inputEnable;
}

//*----------------------------------------------------------------------------
//* \fn    AT91F_PIO_SetOutput
//* \brief Set to 1 output PIO
//*----------------------------------------------------------------------------
static __inline void AT91F_PIO_SetOutput(
	AT91PS_PIO pPio,   // \arg  pointer to a PIO controller
	unsigned int flag) // \arg  output to be set
{
	pPio->PIO_SODR = flag;
}

//*----------------------------------------------------------------------------
//* \fn    AT91F_PIO_ClearOutput
//* \brief Set to 0 output PIO
//*----------------------------------------------------------------------------
static __inline void AT91F_PIO_ClearOutput(
	AT91PS_PIO pPio,   // \arg  pointer to a PIO controller
	unsigned int flag) // \arg  output to be cleared
{
	pPio->PIO_CODR = flag;
}

#endif /* __ATMEL_GPIO_H__*/

