/*	$ JBOSN : hal_interrupt.c, v 0.1 10-12-03 15:12 $     */

/*
 * Copyright (C) 2003	iBOSN systems
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are not permitted completely.
 *
 */

/*
 * Module Name: hal_interrupt.c
 *
 * Purpose: HAL basic operation for interrupt handling
 * HAL_InterruptInit()
 *
 */

#include <types.h>
#include <mach/hal_interrupt.h>

#include <mach/AT91SAM9GX5BD.h>
#include <mach/AT91SAM9GX5.h>

#include <hal_panic.h>
#include <mach/gpio.h>


#define DBG_LVL_IRQ         0
#define DBG_LVL_IRQ_REQ     0
#define DBG_LVL_IRQ_EN      0


/*=================================================================================================
===================================================================================================
                            Interrupt Enable / Disable
===================================================================================================
=================================================================================================*/


/*=================================================================================================
===================================================================================================
                            Interrupt Map Start
===================================================================================================
=================================================================================================*/


#define OAL_INTR_IRQ_UNDEFINED      (-1)

#define OAL_INTR_IRQ_MAXIMUM        IRQ_MAX_COUNT

//------------------------------------------------------------------------------
typedef struct _SYSINTR_2_HALIRQ_
{
    UINT32      Index;
    UINT32      nHalIrq;
}SYSINTR2HALIRQ, *PSYSINTR2HALIRQ;

STATIC SYSINTR2HALIRQ   sSysIntr2HalIrq[SYSINTR_MAXIMUM];
STATIC JB_INTRRUPT      sHalIrq2SysIntr[OAL_INTR_IRQ_MAXIMUM];

STATIC VOID
PIO_InterruptEnable( AT91PS_PIO pPioBase, UINT32 nPinMask )
{
    pPioBase->PIO_ODR = nPinMask; //Output disable
    pPioBase->PIO_PER = nPinMask; //PIO enable (disable peripheral multiplexing)
    pPioBase->PIO_IER = nPinMask; //Interrupt enable
    return;
}



/// Disable the interrupt
STATIC VOID
PIO_InterruptDisable(AT91PS_PIO pPioBase, UINT32 nPinMask )
{
    pPioBase->PIO_IDR = nPinMask;
}

STATIC UINT32
PIO_Interrupt( AT91PS_PIO pBasePio , UINT32 nDeviceID)
{
    AT91PS_AIC  pAIC = (AT91PS_AIC)AT91C_BASE_AIC;
    UINT32      AIC_IMR;
    UINT32      DeviceIDMask;
    UINT32      Status;
    UINT32      nActivatedPin;

    // Following code should not be interrupted by another pio interrupt of the same bank.
    // since critical sections doesn't work here. 
    // just disable the current Pio bank interrupt.
    // (keep trace of the mask register first)

    // Save the Mask register
	AIC_IMR = pAIC->AIC_IMR;

    // Disable current interrupt
	DeviceIDMask = (1<<nDeviceID);
    pAIC->AIC_IDCR = DeviceIDMask;

    //Set the new 'persistent' status for the pio bank (we don't keep the status for interrupt that are not yet enabled)
    Status = (pBasePio->PIO_ISR & pBasePio->PIO_IMR); 
	DEBUGMSG(DBG_LVL_IRQ, "%s (%d): Status = 0x%X, \n", __FUNCTION__, __LINE__,Status);
    for( nActivatedPin = 0; nActivatedPin < 32; nActivatedPin++ )
    {
        if( Status & (1<<nActivatedPin) )
        {
            break;
        }
    }

    if( nActivatedPin < 32 )
    {
        //Mask the pio interrupt for this particular pin, it'll be re-enabled by the user when calling interruptDone.
        pBasePio->PIO_IDR = (1<<nActivatedPin);
    
        //Since we're going to serve this particular interrupt we remove it from the status
        Status &= ~(1<<nActivatedPin); 
    
        // check if the status indicates that another PIO interrupt should be served
        if( Status & pBasePio->PIO_IMR ) 
        {
            //There is still a pio state change pending, force the interrupt to be trigged again
            pAIC->AIC_ISCR = DeviceIDMask;
        }
    }

    // re-Enable the interrupt if it was previously enabled
    pAIC->AIC_IECR = (DeviceIDMask & AIC_IMR);

    return (1<<nActivatedPin);
}

//------------------------------------------------------------------------------
//
//  Function:  OALIntrMapInit
//
//  This function must be called from OALInterruptInit to initialize mapping
//  between IRQ and nSysIntr. It simply initialize mapping arrays.
//
VOID
OALIntrMapInit( VOID )
{
    UINT32 i;

    DEBUGMSG(DBG_LVL_IRQ, "%s (%d):\n", __FUNCTION__, __LINE__);

    // Initialize interrupt maps
    for (i = 0; i < SYSINTR_MAXIMUM; i++)
    {
        sSysIntr2HalIrq[i].Index = 0;
        sSysIntr2HalIrq[i].nHalIrq = OAL_INTR_IRQ_UNDEFINED;
    }

    for (i = 0; i < OAL_INTR_IRQ_MAXIMUM; i++)
    {
        sHalIrq2SysIntr[i].nSysIntr = SYSINTR_NOP;
        sHalIrq2SysIntr[i].BitMap = 0;
    }

    DEBUGMSG(DBG_LVL_IRQ, "%s (%d):\n", __FUNCTION__, __LINE__);
}

/*
//------------------------------------------------------------------------------
//
//  Function:   OALIntrStaticTranslate
//
//  This function sets STATIC translation between IRQ and nSysIntr. In most
//  cases it should not be used. Only exception is mapping for
//  SYSINTR_RTC_ALARM and obsolete device drivers.
//
VOID
OALIntrStaticTranslate(UINT32 nSysIntr, UINT32 irq)
{
    DEBUGMSG(DBG_LVL_IRQ, "+OALIntrStaticTranslate(%d, %d)\n", nSysIntr, irq);

    if ((irq < OAL_INTR_IRQ_MAXIMUM) && (nSysIntr < SYSINTR_MAXIMUM))
    {
        sSysIntr2HalIrq[nSysIntr] = irq;
        
        sHalIrq2SysIntr[irq].nInt = nSysIntr;
        sHalIrq2SysIntr[irq].BitMap = 0;
    }

    DEBUGMSG(DBG_LVL_IRQ, "-OALIntrStaticTranslate\n");
}

//------------------------------------------------------------------------------
//
//  Function:  OALIntrTranslateSysIntr
//
//  This function maps a nSysIntr to its corresponding IRQ. It is typically used
//  in OEMInterruptXXX to obtain IRQs for given nSysIntr.
//
BOOL OALIntrTranslateSysIntr(UINT32 nSysIntr, UINT32 *pCount, CONST JB_PINTERRUPT ppIrqs)
{
    BOOL rc;

    DEBUGMSG(DBG_LVL_IRQ,"+OALTranslateSysIntr(%d)\n", nSysIntr);

    // Valid nSysIntr?
    if (nSysIntr >= SYSINTR_MAXIMUM)
    {
        rc = FALSE;
        goto cleanUp;
    }

    *pCount = 1;
    *ppIrqs = &sSysIntr2HalIrq[nSysIntr];
    rc = TRUE;

cleanUp:

    DEBUGMSG(DBG_LVL_IRQ,"-OALTranslateSysIntr(rc = %d)\n", rc);
    return rc;
}
*/

//------------------------------------------------------------------------------
//
//  Function:  OALIntrTranslateIrq
//
//  This function maps a IRQ to its corresponding nSysIntr.
//
UINT32
OALIntrTranslateIrq(UINT32 nHalIrq)
{
    UINT32 nSysIntr = SYSINTR_NOP;

    DEBUGMSG(DBG_LVL_IRQ, "%s (%d): irq = %d\n", __FUNCTION__, __LINE__, nHalIrq);

    if (nHalIrq >= OAL_INTR_IRQ_MAXIMUM)
        goto cleanUp;

    nSysIntr = sHalIrq2SysIntr[nHalIrq].nSysIntr | (sHalIrq2SysIntr[nHalIrq].BitMap << 16);

    DEBUGMSG(DBG_LVL_IRQ, "%s (%d): nSysIntr = %x\n", __FUNCTION__, __LINE__, nSysIntr);

cleanUp:

    DEBUGMSG(DBG_LVL_IRQ, "%s (%d): nSysIntr = %x\n", __FUNCTION__, __LINE__, nSysIntr);

    return nSysIntr;
}


VOID
OALIntrRequestSystemTimer( UINT32 nHalIrq )
{
    sSysIntr2HalIrq[SYSINTR_TICK].Index   = 0;
    sSysIntr2HalIrq[SYSINTR_TICK].nHalIrq = nHalIrq;

    sHalIrq2SysIntr[nHalIrq].nSysIntr = SYSINTR_TICK;
    sHalIrq2SysIntr[nHalIrq].BitMap = 0;

    return;
}


//------------------------------------------------------------------------------
//
//  Function:  OALIntrRequestSysIntr
//
//  This function allocate new nSysIntr for given IRQ and it there isn't
//  STATIC mapping for this IRQ it will create it.
//
UINT32
OALIntrRequestSysIntr( SYSINTR_PREQUEST pSysIntrReq, PUINT32 pnSysIntr )//UINT32 count, CONST UINT32 *pIrqs, UINT32 flags)
{
    BOOL    bRet = TRUE;
    UINT32  nHalIrq;
    UINT32  nSysIntr;
    UINT32  nSysIntrReq;
    UINT32  BitMap;
    UINT32  Flag;

    nHalIrq  = pSysIntrReq->nHalIrq;
    BitMap   = pSysIntrReq->BitMap;
    Flag     = pSysIntrReq->Flag;

    nSysIntrReq = *pnSysIntr;

//    DEBUGMSG(DBG_LVL_IRQ_REQ, "%s (%d): (%d, %d, 0x%08x)\n", __FUNCTION__, __LINE__, nHalIrq, nSysIntr, Flag);

    // Valid IRQ?
    if( BitMap > 0x8000 || nHalIrq >= OAL_INTR_IRQ_MAXIMUM  )
    {
        nSysIntr = SYSINTR_NOP;
        bRet = FALSE;

        goto cleanUp;
    }
    
    if( nSysIntrReq == 0 )
        Flag &= ~SYSINTR_REQ_FLAG_MULTIPLE;

    // If there is mapping for given irq check for special cases
    if (sHalIrq2SysIntr[nHalIrq].nSysIntr != SYSINTR_NOP)
    {
        // If static mapping is requested we fail
        if( Flag & SYSINTR_REQ_FLAG_STATIC )
        {
            DEBUGMSG(DBG_LVL_IRQ_REQ,"ERROR: OALIntrRequestSysIntr: STATIC mapping for IRQ %d already assigned\n", nHalIrq);
            nSysIntr = SYSINTR_NOP;
            bRet = FALSE;

            goto cleanUp;
        }
        // If we should translate, return existing nSysIntr
        if( Flag & SYSINTR_REQ_FLAG_TRANSLATE )
        {
            nSysIntr = sHalIrq2SysIntr[nHalIrq].nSysIntr;
            bRet = FALSE;

            goto cleanUp;
        }
    }

    // Find next available nSysIntr value...
    for (nSysIntr = SYSINTR_TICK+1; nSysIntr < SYSINTR_MAXIMUM; nSysIntr++)
    {
        if( sSysIntr2HalIrq[nSysIntr].nHalIrq == OAL_INTR_IRQ_UNDEFINED )
            break;
    }

    // Any available SYSINTRs left?
    if( nSysIntr >= SYSINTR_MAXIMUM )
    {
        DEBUGMSG(DBG_LVL_IRQ_REQ,"ERROR: OALIntrRequestSysIntr: No avaiable nSysIntr found\n");
        nSysIntr = SYSINTR_NOP;
        bRet = FALSE;

        goto cleanUp;
    }


    // map the SYSINTR...
    if( Flag & SYSINTR_REQ_FLAG_MULTIPLE )
    {
        UINT32 nSysIntrIndex;

DEBUGMSG(DBG_LVL_IRQ_REQ,"%s (%d) : %d\n", __FUNCTION__, __LINE__, nSysIntrReq);
        if( nSysIntrReq >= SYSINTR_MAXIMUM )
        {
            nSysIntr = SYSINTR_NOP;
            bRet = FALSE;
            goto cleanUp;
        }

        // search the existing 
DEBUGMSG(DBG_LVL_IRQ_REQ,"%s (%d) : %d\n", __FUNCTION__, __LINE__, sSysIntr2HalIrq[nSysIntrReq].nHalIrq);
        if( sSysIntr2HalIrq[nSysIntrReq].nHalIrq == OAL_INTR_IRQ_UNDEFINED )
        {
            nSysIntr = nSysIntrReq;
            bRet = FALSE;
            goto cleanUp;
        }

        //
        nSysIntrIndex = nSysIntrReq;
        do
        {
            UINT32 nSysIntrIndexTemp;
            
            nSysIntrIndexTemp = sSysIntr2HalIrq[nSysIntrIndex].Index;
            if( nSysIntrIndexTemp == 0 )
                break;

            if( nSysIntrIndexTemp >= SYSINTR_MAXIMUM )
            {
                // some error
                nSysIntr = nSysIntrReq;
                bRet = FALSE;
                goto cleanUp;
            }
            nSysIntrIndex = nSysIntrIndexTemp;
        }while(1);

DEBUGMSG(DBG_LVL_IRQ_REQ,"%s (%d) : %d\n", __FUNCTION__, __LINE__, nSysIntrIndex);
DEBUGMSG(DBG_LVL_IRQ_REQ,"%s (%d) : %d\n", __FUNCTION__, __LINE__, nSysIntr);
        sSysIntr2HalIrq[nSysIntrIndex].Index = nSysIntr;

        // Make nSysIntr -> IRQ association.
        sSysIntr2HalIrq[nSysIntr].Index   = 0;
        sSysIntr2HalIrq[nSysIntr].nHalIrq = nHalIrq;

        sHalIrq2SysIntr[nHalIrq].nSysIntr = nSysIntrReq;
        sHalIrq2SysIntr[nHalIrq].BitMap   = BitMap;

        nSysIntr = nSysIntrReq;
    }
    else
    {

        // Make nSysIntr -> IRQ association.
        sSysIntr2HalIrq[nSysIntr].Index   = 0;
        sSysIntr2HalIrq[nSysIntr].nHalIrq = nHalIrq;

        sHalIrq2SysIntr[nHalIrq].nSysIntr = nSysIntr;
        sHalIrq2SysIntr[nHalIrq].BitMap   = BitMap;
    }

//    // Make IRQ -> nSysIntr association if required
//    if( Flag & SYSINTR_REQ_FLAG_DYNAMIC )
//        goto cleanUp;
//
//    if( (sHalIrq2SysIntr[nHalIrq].nInt == SYSINTR_NOP) ||
//        ((Flag & SYSINTR_REQ_FLAG_FORCE_STATIC) != 0) )
//    {
//        sHalIrq2SysIntr[nHalIrq].nInt = nSysIntr;
//        sHalIrq2SysIntr[nHalIrq].BitMap = BitMap;
//    }

cleanUp:
    DEBUGMSG(DBG_LVL_IRQ_REQ, "%s (%d): (nHalIrq = %d, nSysIntr = %d)\n", __FUNCTION__, __LINE__, nHalIrq, 
nSysIntr);

    *pnSysIntr = nSysIntr;

    return bRet;
}

//------------------------------------------------------------------------------
//
//  Function:  OALIntrReleaseSysIntr
//
//  This function release given nSysIntr and remove STATIC mapping if exists.
//
BOOL
OALIntrReleaseSysIntr(UINT32 nSysIntr)
{
    INT32   i;
    BOOL rc = FALSE;
    UINT32 nHalIrq;

    DEBUGMSG(DBG_LVL_IRQ, "%s (%d): (nSysIntr = %d)\n", __FUNCTION__, __LINE__, nSysIntr);

    // is Valid nSysIntr?
    if ((nSysIntr < SYSINTR_TICK+1) || (nSysIntr >= SYSINTR_MAXIMUM))
    {
        DEBUGMSG(DBG_LVL_IRQ,"ERROR: OALIntrReleaseSysIntr: Invalid nSysIntr value %d\n", nSysIntr);
        goto cleanUp;
    }


    for( i = SYSINTR_TICK+1; i < SYSINTR_MAXIMUM ; i++)
    {
        nHalIrq = sSysIntr2HalIrq[i].nHalIrq;
        // Is the nSysIntr already released?
        if( nHalIrq == OAL_INTR_IRQ_UNDEFINED )
            continue;

        // If we're releasing the nSysIntr directly mapped in the IRQ mapping,
        // remove the IRQ mapping also
        if (sHalIrq2SysIntr[nHalIrq].nSysIntr == nSysIntr)
        {
            sSysIntr2HalIrq[i].Index = 0;
            sHalIrq2SysIntr[nHalIrq].nSysIntr = SYSINTR_NOP;
        }
        
        rc = TRUE;
    }

cleanUp:

    DEBUGMSG(DBG_LVL_IRQ, "%s (%d): (rc = %d)\n", __FUNCTION__, __LINE__, rc);

    return rc;
}



//------------------------------------------------------------------------------
//
//  Function:  OALIntrEnableIrqs
//
BOOL OALIntrEnableIrqs( UINT32 nSysIntr )
{
    BOOL bRet = TRUE;
    UINT32 nPhyIrq;
//    UINT32 mask;
    UINT32 Index;
    UINT32 nHalIrq;
    AT91PS_AIC pAIC = (AT91PS_AIC)AT91C_BASE_AIC;
	AT91PS_HDMA			pHDMAReg;

    do
    {
        nHalIrq = sSysIntr2HalIrq[nSysIntr].nHalIrq;
        Index   = sSysIntr2HalIrq[nSysIntr].Index;

        if(nHalIrq == OAL_INTR_IRQ_UNDEFINED)    goto END;

    	// Physical Irq.
    	if(nHalIrq < IRQ_PHY_MAX_COUNT)
    	{
    	    nPhyIrq = nHalIrq;
    	}
    	// PIO INTERRUPT ENABLE
    	else
    	if(nHalIrq >= IRQ_VIR_GPIO_START && nHalIrq < IRQ_VIR_GPIO_END)
    	{
			if(nHalIrq >= IRQ_VIR_GPIO_A_START && nHalIrq < IRQ_VIR_GPIO_B_START)	//PIO A pin interrupt
			{
				PIO_InterruptEnable(AT91C_BASE_PIOA, 1<<(nHalIrq - IRQ_VIR_GPIO_A_START));
				nPhyIrq = AT91C_ID_PIOA_B;
			}
			else if(nHalIrq >= IRQ_VIR_GPIO_B_START && nHalIrq < IRQ_VIR_GPIO_C_START)//PIO B pin interrupt
			{
				PIO_InterruptEnable(AT91C_BASE_PIOB, 1<<(nHalIrq - IRQ_VIR_GPIO_B_START));
				nPhyIrq = AT91C_ID_PIOA_B;
			}
			else if(nHalIrq >= IRQ_VIR_GPIO_C_START && nHalIrq < IRQ_VIR_GPIO_D_START)//PIO C pin interrupt
			{
				PIO_InterruptEnable(AT91C_BASE_PIOC, 1 << (nHalIrq - IRQ_VIR_GPIO_C_START));
				nPhyIrq = AT91C_ID_PIOC_D;
			}
			else if(nHalIrq >= IRQ_VIR_GPIO_D_START && nHalIrq < IRQ_VIR_GPIO_END)//PIO D pin interrupt
			{
				PIO_InterruptEnable(AT91C_BASE_PIOD, 1 <<(nHalIrq - IRQ_VIR_GPIO_D_START));
				nPhyIrq = AT91C_ID_PIOC_D;
			}
    	}
		else if(nHalIrq >= IRQ_VIR_DMA0_START && nHalIrq < IRQ_VIR_DMA0_END)//HDMA0 IRQ
		{
			int DMACh = (nHalIrq - IRQ_VIR_DMA0_START);
			//pHDMAReg = (AT91PS_HDMA)AT91C_BASE_HDMA;
			
			nPhyIrq = AT91C_ID_HDMA;
		}
		else if(nHalIrq >= IRQ_VIR_DMA1_START && nHalIrq < IRQ_VIR_DMA1_END)//HDMA1 IRQ
		{
			int DMACh = (nHalIrq - IRQ_VIR_DMA1_START);
			//pHDMAReg = (AT91PS_HDMA)AT91C_BASE_HDMA1;
			
			nPhyIrq = AT91C_ID_HDMA1;
		}
    	// Physical Irq.
    	else
    	{
    		DEBUGMSG(DBG_LVL_IRQ,"FAIL: OALIntrDisableIrqs, Unknown Irq(%d) \n", nHalIrq);	
    		goto END;
    	}

		pAIC->AIC_IECR = (1 << nPhyIrq ); //eanble AIC interrupt for this controller
		DEBUGMSG(DBG_LVL_IRQ, "%s (%d): (nPhyIrq = %d),(nHalIrq = %d)\n", __FUNCTION__, __LINE__, nPhyIrq,nHalIrq);
        nSysIntr = Index;
        if( nSysIntr >= SYSINTR_MAXIMUM )
            break;

    }while( nSysIntr != 0 );
                 
END:
    DEBUGMSG(DBG_LVL_IRQ, "%s (%d): (rc = %d)\n", __FUNCTION__, __LINE__,bRet);

    return bRet;
}

//------------------------------------------------------------------------------
//
//  Function:  OALIntrDisableIrqs
//
VOID OALIntrDisableIrqs( UINT32 nSysIntr )
{
    UINT32 nPhyIrq;
//    UINT32 mask;

    UINT32 Index;
    UINT32 nHalIrq;
	AT91PS_AIC pAIC = (AT91PS_AIC)AT91C_BASE_AIC;

    do
    {
        Index   = sSysIntr2HalIrq[nSysIntr].Index;
        nHalIrq = sSysIntr2HalIrq[nSysIntr].nHalIrq;

        DEBUGMSG(DBG_LVL_IRQ, "%s (%d): (nSysIntr = %d)\n", __FUNCTION__, __LINE__, nSysIntr);
        DEBUGMSG(DBG_LVL_IRQ, "%s (%d): (nHalIrq = %d)\n", __FUNCTION__, __LINE__, nHalIrq);

        if (nHalIrq == OAL_INTR_IRQ_UNDEFINED)  goto END;

    	// Physical Irq.
    	if(nHalIrq < IRQ_PHY_MAX_COUNT)
    	{
            // Clear INTC pend bit.
            // Disable INTC mask bit.
			nPhyIrq = nHalIrq;
    	}
    	// PIO INTERRUPT DISABLE
    	else
    	if(nHalIrq >= IRQ_VIR_GPIO_START && nHalIrq < IRQ_VIR_GPIO_END)
    	{
			if(nHalIrq >= IRQ_VIR_GPIO_A_START && nHalIrq < IRQ_VIR_GPIO_B_START)	//PIO A pin interrupt
			{
				PIO_InterruptDisable(AT91C_BASE_PIOA, 1<<(nHalIrq - IRQ_VIR_GPIO_A_START));
				nPhyIrq = AT91C_ID_PIOA_B;
			}
			else if(nHalIrq >= IRQ_VIR_GPIO_B_START && nHalIrq < IRQ_VIR_GPIO_C_START)//PIO B pin interrupt
			{
				PIO_InterruptDisable(AT91C_BASE_PIOB, 1<<(nHalIrq - IRQ_VIR_GPIO_B_START));
				nPhyIrq = AT91C_ID_PIOA_B;
			}
			else if(nHalIrq >= IRQ_VIR_GPIO_C_START && nHalIrq < IRQ_VIR_GPIO_D_START)//PIO C pin interrupt
			{
				PIO_InterruptDisable(AT91C_BASE_PIOC, 1<<(nHalIrq - IRQ_VIR_GPIO_C_START));
				nPhyIrq = AT91C_ID_PIOC_D;
			}
			else if(nHalIrq >= IRQ_VIR_GPIO_D_START && nHalIrq < IRQ_VIR_GPIO_END)//PIO D pin interrupt
			{
				PIO_InterruptDisable(AT91C_BASE_PIOD, 1<<(nHalIrq - IRQ_VIR_GPIO_D_START));
				nPhyIrq = AT91C_ID_PIOC_D;
			}
    	}
		else if(nHalIrq >= IRQ_VIR_DMA0_START && nHalIrq < IRQ_VIR_DMA0_END)//HDMA0 IRQ
		{
			int DMACh = (nHalIrq - IRQ_VIR_DMA0_START);
			//pHDMAReg = (AT91PS_HDMA)AT91C_BASE_HDMA;
			
			nPhyIrq = AT91C_ID_HDMA;
		}
		else if(nHalIrq >= IRQ_VIR_DMA1_START && nHalIrq < IRQ_VIR_DMA1_END)//HDMA1 IRQ
		{
			int DMACh = (nHalIrq - IRQ_VIR_DMA1_START);
			//pHDMAReg = (AT91PS_HDMA)AT91C_BASE_HDMA1;
			
			nPhyIrq = AT91C_ID_HDMA1;
		}
		else
        {
            DEBUGMSG(DBG_LVL_IRQ,":( Invalid Interrupt ID !!!!\n");        
        }

        nSysIntr = Index;
		pAIC->AIC_IDCR = (1 << nPhyIrq ); //disable AIC interrupt for this controller
        if( nSysIntr >= SYSINTR_MAXIMUM )
            break;

    }while( nSysIntr != 0 );


END:
    DEBUGMSG(DBG_LVL_IRQ, "%s (%d): \n", __FUNCTION__, __LINE__);
}




//------------------------------------------------------------------------------
//
//  Function:  OALIntrDoneIrqs
//
VOID OALIntrDoneIrqs( UINT32 nSysIntr )
{
    UINT32 nPhyIrq = -1;
//    UINT32 mask;

    UINT32 Index;
    UINT32 nHalIrq;
    AT91PS_AIC pAIC = (AT91PS_AIC)AT91C_BASE_AIC;

    do
    {
        Index   = sSysIntr2HalIrq[nSysIntr].Index;
        nHalIrq = sSysIntr2HalIrq[nSysIntr].nHalIrq;

        DEBUGMSG(DBG_LVL_IRQ, "%s (%d): (nSysIntr = %d)\n", __FUNCTION__, __LINE__, nSysIntr);
        DEBUGMSG(DBG_LVL_IRQ, "%s (%d): (nHalIrq = %d)\n", __FUNCTION__, __LINE__, nHalIrq);

    	// Physical Irq.
    	if(nHalIrq < IRQ_PHY_MAX_COUNT)
    	{	
    	    nPhyIrq = nHalIrq;
    	}
    	// PIO INTERRUPT ENABLE
    	else
    	if(nHalIrq >= IRQ_VIR_GPIO_START && nHalIrq < IRQ_VIR_GPIO_END)
    	{
			if(nHalIrq >= IRQ_VIR_GPIO_A_START && nHalIrq < IRQ_VIR_GPIO_B_START)	//PIO A pin interrupt
			{
				PIO_InterruptEnable(AT91C_BASE_PIOA, 1<<(nHalIrq - IRQ_VIR_GPIO_A_START));
				nPhyIrq = AT91C_ID_PIOA_B;
			}
			else if(nHalIrq >= IRQ_VIR_GPIO_B_START && nHalIrq < IRQ_VIR_GPIO_C_START)//PIO B pin interrupt
			{
				PIO_InterruptEnable(AT91C_BASE_PIOB, 1<<(nHalIrq - IRQ_VIR_GPIO_B_START));
				nPhyIrq = AT91C_ID_PIOA_B;
				
			}
			else if(nHalIrq >= IRQ_VIR_GPIO_C_START && nHalIrq < IRQ_VIR_GPIO_D_START)//PIO C pin interrupt
			{
				PIO_InterruptEnable(AT91C_BASE_PIOC, 1<<(nHalIrq - IRQ_VIR_GPIO_C_START));
				nPhyIrq = AT91C_ID_PIOC_D;
			}
			else if(nHalIrq >= IRQ_VIR_GPIO_D_START && nHalIrq < IRQ_VIR_GPIO_END)//PIO D pin interrupt
			{
				PIO_InterruptEnable(AT91C_BASE_PIOD, 1<<(nHalIrq - IRQ_VIR_GPIO_D_START));
				nPhyIrq = AT91C_ID_PIOC_D;
			}
    	}
		else if(nHalIrq >= IRQ_VIR_DMA0_START && nHalIrq < IRQ_VIR_DMA0_END)//HDMA0 IRQ
		{
			int DMACh = (nHalIrq - IRQ_VIR_DMA0_START);
			//pHDMAReg = (AT91PS_HDMA)AT91C_BASE_HDMA;
			
			nPhyIrq = AT91C_ID_HDMA;
		}
		else if(nHalIrq >= IRQ_VIR_DMA1_START && nHalIrq < IRQ_VIR_DMA1_END)//HDMA1 IRQ
		{
			int DMACh = (nHalIrq - IRQ_VIR_DMA1_START);
			//pHDMAReg = (AT91PS_HDMA)AT91C_BASE_HDMA1;
			
			nPhyIrq = AT91C_ID_HDMA1;
		}
    	// Physical Irq.
    	else
    	{
            DEBUGMSG(DBG_LVL_IRQ,"FAIL: OALIntrDoneIrqs, Unknown nPhyIrq(%d)\n", nPhyIrq);
            goto END;
    	}

    	// Set Interrupt controller.
        pAIC->AIC_IECR = (1 << nPhyIrq ); //eanble AIC interrupt for this controller


        nSysIntr = Index;
        if( nSysIntr >= SYSINTR_MAXIMUM )
            break;

    }while( nSysIntr != 0 );

END:
    DEBUGMSG(DBG_LVL_IRQ,"%s : \n", __FUNCTION__ );
}



/*=================================================================================================
===================================================================================================
                    Interrupt Map End
===================================================================================================
=================================================================================================*/

//
//
//STATIC void PrepareEINTIntr(void)
//{
//    // Wakeup source EINT(GPH) Initialization
//    OUTREG32(&g_pGPIOReg->EINTMSK.EXT_INT0_MASK, 0xFF);
//    OUTREG32(&g_pGPIOReg->EINTMSK.EXT_INT1_MASK, 0xFF);
//    OUTREG32(&g_pGPIOReg->EINTMSK.EXT_INT2_MASK, 0xFF);
//    OUTREG32(&g_pGPIOReg->EINTMSK.EXT_INT3_MASK, 0xFF);
//    OUTREG32(&g_pGPIOReg->EINTPND.EXT_INT0_PEND, 0xFF);
//    OUTREG32(&g_pGPIOReg->EINTPND.EXT_INT1_PEND, 0xFF);
//    OUTREG32(&g_pGPIOReg->EINTPND.EXT_INT2_PEND, 0xFF);
//    OUTREG32(&g_pGPIOReg->EINTPND.EXT_INT3_PEND, 0xFF);
//
//    // General EINT Initialization
//    OUTREG32(&g_pGPIOReg->GPIOINTMSK.GPA0_INT_MASK, 0xFF);
//    OUTREG32(&g_pGPIOReg->GPIOINTMSK.GPA1_INT_MASK, 0xFF);
//    OUTREG32(&g_pGPIOReg->GPIOINTMSK.GPB_INT_MASK, 0xFF);
//    OUTREG32(&g_pGPIOReg->GPIOINTMSK.GPC0_INT_MASK, 0xFF);
//    OUTREG32(&g_pGPIOReg->GPIOINTMSK.GPC1_INT_MASK, 0xFF);
//    OUTREG32(&g_pGPIOReg->GPIOINTMSK.GPD0_INT_MASK, 0xFF);
//    OUTREG32(&g_pGPIOReg->GPIOINTMSK.GPD1_INT_MASK, 0xFF);
//    OUTREG32(&g_pGPIOReg->GPIOINTMSK.GPE0_INT_MASK, 0xFF);
//    OUTREG32(&g_pGPIOReg->GPIOINTMSK.GPE1_INT_MASK, 0xFF);
//    OUTREG32(&g_pGPIOReg->GPIOINTMSK.GPF0_INT_MASK, 0xFF);
//    OUTREG32(&g_pGPIOReg->GPIOINTMSK.GPF1_INT_MASK, 0xFF);
//    OUTREG32(&g_pGPIOReg->GPIOINTMSK.GPF2_INT_MASK, 0xFF);
//    OUTREG32(&g_pGPIOReg->GPIOINTMSK.GPF3_INT_MASK, 0xFF);
//    OUTREG32(&g_pGPIOReg->GPIOINTMSK.GPG0_INT_MASK, 0xFF);
//    OUTREG32(&g_pGPIOReg->GPIOINTMSK.GPG1_INT_MASK, 0xFF);
//    OUTREG32(&g_pGPIOReg->GPIOINTMSK.GPG2_INT_MASK, 0xFF);
//    OUTREG32(&g_pGPIOReg->GPIOINTMSK.GPG3_INT_MASK, 0xFF);
//    OUTREG32(&g_pGPIOReg->GPIOINTMSK.GPJ0_INT_MASK, 0xFF);
//    OUTREG32(&g_pGPIOReg->GPIOINTMSK.GPJ1_INT_MASK, 0xFF);
//    OUTREG32(&g_pGPIOReg->GPIOINTMSK.GPJ2_INT_MASK, 0xFF);
//    OUTREG32(&g_pGPIOReg->GPIOINTMSK.GPJ3_INT_MASK, 0xFF);
//
//    OUTREG32(&g_pGPIOReg->GPIOINTPND.GPA0_INT_PEND, 0xFF);
//    OUTREG32(&g_pGPIOReg->GPIOINTPND.GPA1_INT_PEND, 0xFF);
//    OUTREG32(&g_pGPIOReg->GPIOINTPND.GPB_INT_PEND, 0xFF);
//    OUTREG32(&g_pGPIOReg->GPIOINTPND.GPC0_INT_PEND, 0xFF);
//    OUTREG32(&g_pGPIOReg->GPIOINTPND.GPC1_INT_PEND, 0xFF);
//    OUTREG32(&g_pGPIOReg->GPIOINTPND.GPD0_INT_PEND, 0xFF);
//    OUTREG32(&g_pGPIOReg->GPIOINTPND.GPD1_INT_PEND, 0xFF);
//    OUTREG32(&g_pGPIOReg->GPIOINTPND.GPE0_INT_PEND, 0xFF);
//    OUTREG32(&g_pGPIOReg->GPIOINTPND.GPE1_INT_PEND, 0xFF);
//    OUTREG32(&g_pGPIOReg->GPIOINTPND.GPF0_INT_PEND, 0xFF);
//    OUTREG32(&g_pGPIOReg->GPIOINTPND.GPF1_INT_PEND, 0xFF);
//    OUTREG32(&g_pGPIOReg->GPIOINTPND.GPF2_INT_PEND, 0xFF);
//    OUTREG32(&g_pGPIOReg->GPIOINTPND.GPF3_INT_PEND, 0xFF);
//    OUTREG32(&g_pGPIOReg->GPIOINTPND.GPG0_INT_PEND, 0xFF);
//    OUTREG32(&g_pGPIOReg->GPIOINTPND.GPG1_INT_PEND, 0xFF);
//    OUTREG32(&g_pGPIOReg->GPIOINTPND.GPG2_INT_PEND, 0xFF);
//    OUTREG32(&g_pGPIOReg->GPIOINTPND.GPG3_INT_PEND, 0xFF);
//    OUTREG32(&g_pGPIOReg->GPIOINTPND.GPJ0_INT_PEND, 0xFF);
//    OUTREG32(&g_pGPIOReg->GPIOINTPND.GPJ1_INT_PEND, 0xFF);
//    OUTREG32(&g_pGPIOReg->GPIOINTPND.GPJ2_INT_PEND, 0xFF);
//    OUTREG32(&g_pGPIOReg->GPIOINTPND.GPJ3_INT_PEND, 0xFF);
//}
//
//STATIC void PrepareDMACIntr(void)
//{
//    // Disable Interrupt of All Channel
//    // Mask TC and Error Interrupt
//    // Clear Interrupt Pending
//
//#if 0
//    g_pDMAC0Reg->DMACC0Control0 &= ~(1<<31);        // TCINT Disable
//    g_pDMAC0Reg->DMACC0Configuration &= (3<<14);    // TCINT & ErrINT Mask
//    g_pDMAC0Reg->DMACC1Control0 &= ~(1<<31);        // TCINT Disable
//    g_pDMAC0Reg->DMACC1Configuration &= (3<<14);    // TCINT & ErrINT Mask
//    g_pDMAC0Reg->DMACC2Control0 &= ~(1<<31);        // TCINT Disable
//    g_pDMAC0Reg->DMACC2Configuration &= (3<<14);    // TCINT & ErrINT Mask
//    g_pDMAC0Reg->DMACC3Control0 &= ~(1<<31);        // TCINT Disable
//    g_pDMAC0Reg->DMACC3Configuration &= (3<<14);    // TCINT & ErrINT Mask
//    g_pDMAC0Reg->DMACC4Control0 &= ~(1<<31);        // TCINT Disable
//    g_pDMAC0Reg->DMACC4Configuration &= (3<<14);    // TCINT & ErrINT Mask
//    g_pDMAC0Reg->DMACC5Control0 &= ~(1<<31);        // TCINT Disable
//    g_pDMAC0Reg->DMACC5Configuration &= (3<<14);    // TCINT & ErrINT Mask
//    g_pDMAC0Reg->DMACC6Control0 &= ~(1<<31);        // TCINT Disable
//    g_pDMAC0Reg->DMACC6Configuration &= (3<<14);    // TCINT & ErrINT Mask
//    g_pDMAC0Reg->DMACC7Control0 &= ~(1<<31);        // TCINT Disable
//    g_pDMAC0Reg->DMACC7Configuration &= (3<<14);    // TCINT & ErrINT Mask
//
//    g_pDMAC0Reg->DMACIntTCClear = 0xff;            // TC Int Pending Clear
//    g_pDMAC0Reg->DMACIntErrClear = 0xff;            // Err Int Pending Clear
//
//    g_pDMAC1Reg->DMACC0Control0 &= ~(1<<31);        // TCINT Disable
//    g_pDMAC1Reg->DMACC0Configuration &= (3<<14);    // TCINT & ErrINT Mask
//    g_pDMAC1Reg->DMACC1Control0 &= ~(1<<31);        // TCINT Disable
//    g_pDMAC1Reg->DMACC1Configuration &= (3<<14);    // TCINT & ErrINT Mask
//    g_pDMAC1Reg->DMACC2Control0 &= ~(1<<31);        // TCINT Disable
//    g_pDMAC1Reg->DMACC2Configuration &= (3<<14);    // TCINT & ErrINT Mask
//    g_pDMAC1Reg->DMACC3Control0 &= ~(1<<31);        // TCINT Disable
//    g_pDMAC1Reg->DMACC3Configuration &= (3<<14);    // TCINT & ErrINT Mask
//    g_pDMAC1Reg->DMACC4Control0 &= ~(1<<31);        // TCINT Disable
//    g_pDMAC1Reg->DMACC4Configuration &= (3<<14);    // TCINT & ErrINT Mask
//    g_pDMAC1Reg->DMACC5Control0 &= ~(1<<31);        // TCINT Disable
//    g_pDMAC1Reg->DMACC5Configuration &= (3<<14);    // TCINT & ErrINT Mask
//    g_pDMAC1Reg->DMACC6Control0 &= ~(1<<31);        // TCINT Disable
//    g_pDMAC1Reg->DMACC6Configuration &= (3<<14);    // TCINT & ErrINT Mask
//    g_pDMAC1Reg->DMACC7Control0 &= ~(1<<31);        // TCINT Disable
//    g_pDMAC1Reg->DMACC7Configuration &= (3<<14);    // TCINT & ErrINT Mask
//
//    g_pDMAC1Reg->DMACIntTCClear = 0xff;            // TC Int Pending Clear
//    g_pDMAC1Reg->DMACIntErrClear = 0xff;            // Err Int Pending Clear
//#endif
//
//}
//
//
//
//void VICTableInit(void)
//{
//    // This Function is reference by OEMPowerOff() in "Off.c"
//    // Make Sure that Caller and This function is in Same Address Space
//
//    // Fill Vector Address of VIC0
//    // Actually, Filled with Physical IRQ Numbers.
//    // Because We do not use vectored interrupt feature
//
//
//    // VIC0
//    g_pVIC0Reg->VICVECTADDR0 = PHYIRQ_EINT0;        // 0          // 0
//    g_pVIC0Reg->VICVECTADDR1 = PHYIRQ_EINT1;        // 1          // 1
//    g_pVIC0Reg->VICVECTADDR2 = PHYIRQ_EINT2;        // 2          // 2
//    g_pVIC0Reg->VICVECTADDR3 = PHYIRQ_EINT3;        // 3          // 3
//    g_pVIC0Reg->VICVECTADDR4 = PHYIRQ_EINT4;        // 4          // 4
//    g_pVIC0Reg->VICVECTADDR5 = PHYIRQ_EINT5;        // 5          // 5
//    g_pVIC0Reg->VICVECTADDR6 = PHYIRQ_EINT6;        // 6          // 6
//    g_pVIC0Reg->VICVECTADDR7 = PHYIRQ_EINT7;        // 7          // 7
//    g_pVIC0Reg->VICVECTADDR8 = PHYIRQ_EINT8;        // 8          // 8
//    g_pVIC0Reg->VICVECTADDR9 = PHYIRQ_EINT9;        // 9          // 9
//    g_pVIC0Reg->VICVECTADDR10 = PHYIRQ_EINT10;       // 10         // 10
//    g_pVIC0Reg->VICVECTADDR11 = PHYIRQ_EINT11;       // 11         // 11
//    g_pVIC0Reg->VICVECTADDR12 = PHYIRQ_EINT12;       // 12         // 12
//    g_pVIC0Reg->VICVECTADDR13 = PHYIRQ_EINT13;       // 13         // 13
//    g_pVIC0Reg->VICVECTADDR14 = PHYIRQ_EINT14;       // 14         // 14
//    g_pVIC0Reg->VICVECTADDR15 = PHYIRQ_EINT15;       // 15         // 15
//    g_pVIC0Reg->VICVECTADDR16 = PHYIRQ_EINT16_31;    // 16         // 16
//    g_pVIC0Reg->VICVECTADDR17 = PHYIRQ_RESERVED0;    // 32         // 17  
//    g_pVIC0Reg->VICVECTADDR18 = PHYIRQ_MDMA;         // 33         // 18
//    g_pVIC0Reg->VICVECTADDR19 = PHYIRQ_PDMA0;        // 41         // 19
//    g_pVIC0Reg->VICVECTADDR20 = PHYIRQ_PDMA1;        // 49         // 20
//    g_pVIC0Reg->VICVECTADDR21 = PHYIRQ_TIMER0;       // 57         // 21
//    g_pVIC0Reg->VICVECTADDR22 = PHYIRQ_TIMER1;       // 58         // 22
//    g_pVIC0Reg->VICVECTADDR23 = PHYIRQ_TIMER2;       // 59         // 23
//    g_pVIC0Reg->VICVECTADDR24 = PHYIRQ_TIMER3;       // 60         // 24
//    g_pVIC0Reg->VICVECTADDR25 = PHYIRQ_TIMER4;       // 61         // 25
//    g_pVIC0Reg->VICVECTADDR26 = PHYIRQ_SYSTIMER;     // 62         // 26
//    g_pVIC0Reg->VICVECTADDR27 = PHYIRQ_WDT;          // 63         // 27
//    g_pVIC0Reg->VICVECTADDR28 = PHYIRQ_RTC_ALARM;    // 64         // 28
//    g_pVIC0Reg->VICVECTADDR29 = PHYIRQ_RTC_TIC;      // 65         // 29
//    g_pVIC0Reg->VICVECTADDR30 = PHYIRQ_GPIO;         // 66         // 30
//    g_pVIC0Reg->VICVECTADDR31 = PHYIRQ_FIMC3;        // 67         // 31
//
//    // VIC1
//    g_pVIC1Reg->VICVECTADDR0 = PHYIRQ_CORTEX0;      // 68         // 32
//    g_pVIC1Reg->VICVECTADDR1 = PHYIRQ_CORTEX1;      // 69         // 33
//    g_pVIC1Reg->VICVECTADDR2 = PHYIRQ_CORTEX2;      // 70         // 34
//    g_pVIC1Reg->VICVECTADDR3 = PHYIRQ_CORTEX3;      // 71         // 35
//    g_pVIC1Reg->VICVECTADDR4 = PHYIRQ_CORTEX4;      // 72         // 36
//    g_pVIC1Reg->VICVECTADDR5 = PHYIRQ_IEMAPC;       // 73         // 37
//    g_pVIC1Reg->VICVECTADDR6 = PHYIRQ_IEMIEC;       // 74         // 38
//    g_pVIC1Reg->VICVECTADDR7 = PHYIRQ_RESERVED1;    // 75         // 39
//    g_pVIC1Reg->VICVECTADDR8 = PHYIRQ_NFC;          // 76         // 40
//    g_pVIC1Reg->VICVECTADDR9 = PHYIRQ_CFC;          // 77         // 41
//    g_pVIC1Reg->VICVECTADDR10 = PHYIRQ_UART0;        // 78         // 42
//    g_pVIC1Reg->VICVECTADDR11 = PHYIRQ_UART1;        // 79         // 43
//    g_pVIC1Reg->VICVECTADDR12 = PHYIRQ_UART2;        // 80         // 44
//    g_pVIC1Reg->VICVECTADDR13 = PHYIRQ_UART3;        // 81         // 45
//    g_pVIC1Reg->VICVECTADDR14 = PHYIRQ_I2C;          // 82         // 46
//    g_pVIC1Reg->VICVECTADDR15 = PHYIRQ_SPI0;         // 83         // 47
//    g_pVIC1Reg->VICVECTADDR16 = PHYIRQ_SPI1;         // 84         // 48
//    g_pVIC1Reg->VICVECTADDR17 = PHYIRQ_SPI2;         // 85         // 49
//    g_pVIC1Reg->VICVECTADDR18 = PHYIRQ_AUDIO_SS;     // 86         // 50
//    g_pVIC1Reg->VICVECTADDR19 = PHYIRQ_I2C_PMIC_LINK;// 87         // 51
//    g_pVIC1Reg->VICVECTADDR20 = PHYIRQ_I2C_HDMI_PHY; // 88         // 52
//    g_pVIC1Reg->VICVECTADDR21 = PHYIRQ_HSIRX;        // 89         // 53
//    g_pVIC1Reg->VICVECTADDR22 = PHYIRQ_HSITX;        // 90         // 54
//    g_pVIC1Reg->VICVECTADDR23 = PHYIRQ_UHOST;        // 91         // 55
//    g_pVIC1Reg->VICVECTADDR24 = PHYIRQ_OTG;          // 92         // 56
//    g_pVIC1Reg->VICVECTADDR25 = PHYIRQ_MSM;          // 93         // 57
//    g_pVIC1Reg->VICVECTADDR26 = PHYIRQ_SDMMC0;       // 94         // 58
//    g_pVIC1Reg->VICVECTADDR27 = PHYIRQ_SDMMC1;       // 95         // 59
//    g_pVIC1Reg->VICVECTADDR28 = PHYIRQ_SDMMC2;       // 96         // 60
//    g_pVIC1Reg->VICVECTADDR29 = PHYIRQ_MIPI_CSI;     // 97         // 61
//    g_pVIC1Reg->VICVECTADDR30 = PHYIRQ_MIPI_DSI;     // 98         // 62
//    g_pVIC1Reg->VICVECTADDR31 = PHYIRQ_ONENAND_AUDI; // 99         // 63
//
//    // VIC2
//    g_pVIC2Reg->VICVECTADDR0 = PHYIRQ_LCD0;         // 100        // 64
//    g_pVIC2Reg->VICVECTADDR1 = PHYIRQ_LCD1;         // 101        // 65
//    g_pVIC2Reg->VICVECTADDR2 = PHYIRQ_LCD2;         // 102        // 66
//    g_pVIC2Reg->VICVECTADDR3 = PHYIRQ_LCD3;         // 103        // 67
//    g_pVIC2Reg->VICVECTADDR4 = PHYIRQ_ROTATOR;      // 104        // 68
//    g_pVIC2Reg->VICVECTADDR5 = PHYIRQ_FIMC0;        // 105        // 69
//    g_pVIC2Reg->VICVECTADDR6 = PHYIRQ_FIMC1;        // 106        // 70
//    g_pVIC2Reg->VICVECTADDR7 = PHYIRQ_FIMC2;        // 107        // 71
//    g_pVIC2Reg->VICVECTADDR8 = PHYIRQ_JPEG;         // 108        // 72
//    g_pVIC2Reg->VICVECTADDR9 = PHYIRQ_2D;           // 109        // 73
//    g_pVIC2Reg->VICVECTADDR10 = PHYIRQ_3D;           // 110        // 74
//    g_pVIC2Reg->VICVECTADDR11 = PHYIRQ_MIXER;        // 111        // 75
//    g_pVIC2Reg->VICVECTADDR12 = PHYIRQ_HDMI;         // 112        // 76
//    g_pVIC2Reg->VICVECTADDR13 = PHYIRQ_HDMI_I2C;     // 113        // 77
//    g_pVIC2Reg->VICVECTADDR14 = PHYIRQ_MFC;          // 114        // 78
//    g_pVIC2Reg->VICVECTADDR15 = PHYIRQ_TVENC;        // 115        // 79
//    g_pVIC2Reg->VICVECTADDR16 = PHYIRQ_I2S0;         // 116        // 80
//    g_pVIC2Reg->VICVECTADDR17 = PHYIRQ_I2S1;         // 117        // 81
//    g_pVIC2Reg->VICVECTADDR18 = PHYIRQ_RESERVED2;    // 118        // 82 
//    g_pVIC2Reg->VICVECTADDR19 = PHYIRQ_AC97;         // 119        // 83
//    g_pVIC2Reg->VICVECTADDR20 = PHYIRQ_PCM0;         // 120        // 84
//    g_pVIC2Reg->VICVECTADDR21 = PHYIRQ_PCM1;         // 121        // 85
//    g_pVIC2Reg->VICVECTADDR22 = PHYIRQ_SPDIF;        // 122        // 86
//    g_pVIC2Reg->VICVECTADDR23 = PHYIRQ_ADC;          // 123        // 87
//    g_pVIC2Reg->VICVECTADDR24 = PHYIRQ_PENDN;        // 124        // 88
//    g_pVIC2Reg->VICVECTADDR25 = PHYIRQ_KEYPAD;       // 125        // 89
//    g_pVIC2Reg->VICVECTADDR26 = PHYIRQ_SPG;          // 126        // 90
//    g_pVIC2Reg->VICVECTADDR27 = PHYIRQ_IntHash_SSS;  // 127        // 91
//    g_pVIC2Reg->VICVECTADDR28 = PHYIRQ_IntFeedCtrl_SSS;       // 128        // 92
//    g_pVIC2Reg->VICVECTADDR29 = PHYIRQ_RESERVED3;    // 129        // 93
//    g_pVIC2Reg->VICVECTADDR30 = PHYIRQ_SDM_IRQ;      // 130        // 94
//    g_pVIC2Reg->VICVECTADDR31 = PHYIRQ_SDM_FIQ;      // 131        // 95
//    
//    // VIC3
//    g_pVIC3Reg->VICVECTADDR0 = PHYIRQ_IPC;           // 132        // 96
//    g_pVIC3Reg->VICVECTADDR1 = PHYIRQ_HOSTIF;        // 133        // 97
//    g_pVIC3Reg->VICVECTADDR2 = PHYIRQ_SDMMC3;        // 134        // 98
//    g_pVIC3Reg->VICVECTADDR3 = PHYIRQ_CEC;           // 135        // 99
//    g_pVIC3Reg->VICVECTADDR4 = PHYIRQ_TSI;           // 136        // 100
//    g_pVIC3Reg->VICVECTADDR5 = PHYIRQ_MDNIE0;        // 137        // 101
//    g_pVIC3Reg->VICVECTADDR6 = PHYIRQ_MDNIE1;        // 138        // 102
//    g_pVIC3Reg->VICVECTADDR7 = PHYIRQ_MDNIE2;        // 139        // 103
//    g_pVIC3Reg->VICVECTADDR8 = PHYIRQ_MDNIE3;        // 140        // 104    
//    g_pVIC3Reg->VICVECTADDR9 = PHYIRQ_ADC1;          // 141        // 105    
//    g_pVIC3Reg->VICVECTADDR10 = PHYIRQ_PENDN1;       // 142        // 106    
//}
//
//
//
//
//STATIC void InitializeVIC(void)
//{
//    // Disable All Interrupts
//    g_pVIC0Reg->VICINTENCLEAR = 0xFFFFFFFF;
//    g_pVIC1Reg->VICINTENCLEAR = 0xFFFFFFFF;
//    g_pVIC2Reg->VICINTENCLEAR = 0xFFFFFFFF;
//    g_pVIC3Reg->VICINTENCLEAR = 0xFFFFFFFF;
//    g_pVIC0Reg->VICSOFTINTCLEAR = 0xFFFFFFFF;
//    g_pVIC1Reg->VICSOFTINTCLEAR = 0xFFFFFFFF;
//    g_pVIC2Reg->VICSOFTINTCLEAR = 0xFFFFFFFF;
//    g_pVIC3Reg->VICSOFTINTCLEAR = 0xFFFFFFFF;
//
//    // Clear Current Active Vector Address
//    g_pVIC0Reg->VICADDRESS = 0x0;
//    g_pVIC1Reg->VICADDRESS = 0x0;
//    g_pVIC2Reg->VICADDRESS = 0x0;
//    g_pVIC3Reg->VICADDRESS = 0x0;
//
//    // System Timer Interrupt is highest priority
//    g_pVIC0Reg->VICVECTPRIORITY26 = (g_pVIC0Reg->VICVECTPRIORITY26 & ~(0xf))|0x0;
//    
//    // Initialize Vector Address Table
//    VICTableInit();
//
//    // Enable IRQ Interrupt on CP15
//    System_EnableIRQ();
//
//    //// Enable FIQ Interrupt on CP15
//    //System_EnableFIQ();
//}


/*
  Name: HAL_InterruptInit
  Copyright: 
  Author: jung byoung oh
  Date: 10-12-03 15:05
  Description: Initialize hardware specific interrupt controller
*/
PUBLIC VOID
HAL_InterruptInit( VOID )
{
    INT32       i;
    UINT32      Temp;
    AT91PS_PIO  pPioBase;
	AT91PS_AIC  pAIC = (AT91PS_AIC)AT91C_BASE_AIC;
	AT91PS_PMC  pPMC = (AT91PS_PMC)AT91C_BASE_PMC;

    // Disable all interrupts
    pAIC->AIC_IDCR = -1;

    // clear all interrupts
    pAIC->AIC_ICCR = -1;
    
    // Configure the AIC to correctly manage the interrupts from the peripheral controllers.
    // Later the user would be able to change those properties by calling a kernelIoControl (IOCTL_HAL_SET_IRQ_PROPERTIES -not implemented yet-)
    // in order for example to handle interrupt on IRQ0-IRQ6
    for (i = 0; i <= LOGINTR_AIC_LAST; i++) 
    {
		// Create an IRQ descriptor for each AIC entry
		// Init each SMR AIC entry
		pAIC->AIC_SVR[i] = i;
		pAIC->AIC_SMR[i] = AT91C_AIC_PRIOR_LOWEST | AT91C_AIC_SRCTYPE_INT_LEVEL_SENSITIVE;

		// Perform 8 End Of Interrupt Command to make sure AIC will not Lock out nIRQ
		if (i < 8)
			pAIC->AIC_EOICR = pAIC->AIC_EOICR;
    }

    // Initialize the PIO interrupt engine

    // port D~E
    pPioBase = (AT91PS_PIO)AT91C_BASE_PIOD;
    pPMC->PMC_PCER = (1 << AT91C_ID_PIOC_D );       //power up the controller
    pPioBase->PIO_IDR = 0xFFFFFFFF;                 //disable all interrupts from this bank		
    Temp = pPioBase->PIO_ISR;                       //clear any pending pio interrupt		
    pAIC->AIC_IECR = (1<<AT91C_ID_PIOC_D);          //eanble AIC interrupt for this controller		
/*
    pPioBase = (AT91PS_PIO)AT91C_BASE_PIOE;
    pPMC->PMC_PCER = (1 << AT91C_ID_PIOD_E );       //power up the controller
    pPioBase->PIO_IDR = 0xFFFFFFFF;                 //disable all interrupts from this bank		
    Temp = pPioBase->PIO_ISR;                       //clear any pending pio interrupt		
    pAIC->AIC_IECR = (1<<AT91C_ID_PIOD_E);          //eanble AIC interrupt for this controller		
*/
    // port C
    pPioBase = (AT91PS_PIO)AT91C_BASE_PIOC;
    pPMC->PMC_PCER = (1 << AT91C_ID_PIOC_D );        //power up the controller
    pPioBase->PIO_IDR = 0xFFFFFFFF;                 //disable all interrupts from this bank		
    Temp = pPioBase->PIO_ISR;                       //clear any pending pio interrupt		
    pAIC->AIC_IECR = (1<<AT91C_ID_PIOC_D);            //eanble AIC interrupt for this controller		

    // port B
    pPioBase = (AT91PS_PIO)AT91C_BASE_PIOB;
    pPMC->PMC_PCER = (1 << AT91C_ID_PIOA_B );     //power up the controller
    pPioBase->PIO_IDR = 0xFFFFFFFF;                 //disable all interrupts from this bank		
    Temp = pPioBase->PIO_ISR;                       //clear any pending pio interrupt		
    pAIC->AIC_IECR = (1<<AT91C_ID_PIOA_B);            //eanble AIC interrupt for this controller		

    // port A
    pPioBase = (AT91PS_PIO)AT91C_BASE_PIOA;
    pPMC->PMC_PCER = (1 << AT91C_ID_PIOA_B );     //power up the controller
    pPioBase->PIO_IDR = 0xFFFFFFFF;                 //disable all interrupts from this bank		
    Temp = pPioBase->PIO_ISR;                       //clear any pending pio interrupt		
    pAIC->AIC_IECR = (1<<AT91C_ID_PIOA_B);            //eanble AIC interrupt for this controller		

	// Initialize interrupt mapping
    OALIntrMapInit();

    DEBUGMSG(DBG_LVL_IRQ, "%s (%d): --\n", __FUNCTION__, __LINE__);

    return;

}




/*
  Name: HAL_InterruptHandler()
  Copyright: 
  Author: jung byoung oh
  Date: 10-12-03 15:06
  Description: hardware interrupt processing handler
      Return : virtual interrupt number.
*/
PUBLIC UINT32	
HAL_InterruptHandler( VOID )
{
    UINT32  nIVR;
    //UINT32  iSysIntr = SYSINTR_NOP;
    UINT32 nSysIntr = SYSINTR_NOP;
    UINT32 nPhyIrq = 0;
    UINT32 nHalIrq = -1;

    AT91PS_AIC  pAIC = (AT91PS_AIC)AT91C_BASE_AIC;
	AT91PS_SYS pSYS = (AT91PS_SYS)AT91C_BASE_SYS;

	nIVR = pAIC->AIC_IVR;

	// first aknowledge the interrupt at the AIC level
	pAIC->AIC_EOICR = pAIC->AIC_EOICR;

#if 1
    if( nIVR == 0 )
        return nSysIntr;
#endif

    #if TIMER_LED_DISPLAY
    STATIC DWORD HeartBeatCnt, HeartBeatStat;  //LED4 is used for heart beat
    #endif  // TIMER_LED_DISPLAY
    // then check if it's a system controller interrupt
    if( nIVR == AT91C_ID_SYS )
    {
		/// if a System Timer Interrupt occured, then call the timer interrupt rr
        if( pSYS->SYS_PITC_PISR & AT91C_PITC_PITS )
        {
			DEBUGMSG(0,"AT91C_ID_SYS(SYSINTR_TICK)\n", nIVR);
            nSysIntr = SYSINTR_TICK;
        }
    }
    else
    {
#if 0
        {
        	UINT32 i;

        	for( i=2 ; i<32 ; i++ )
        	{
        		if ((nIVR & (1<<i)))
        		{
        		    nPhyIrq += i;
                    break;
        		}
        	}
        }
#endif
		nPhyIrq = nIVR;

		if (0 == nPhyIrq)
		{
			DEBUGMSG(DBG_LVL_IRQ,"%s(%d) nPhyIrq == 0\n",__FUNCTION__,__LINE__);
        }
		else if(AT91C_ID_PIOA_B == nPhyIrq || AT91C_ID_PIOC_D == nPhyIrq)
		{
			UINT32  ActivatedBit = 0;
			UINT32	i=0;
			if(ActivatedBit = PIO_Interrupt(AT91C_BASE_PIOA, AT91C_ID_PIOA_B))
			{
				for(i=0;i<32;i++)
				{
					if((ActivatedBit >> i) == 0x01)
						break;
				}
				if(i < 32)
				{
					nHalIrq = IRQ_VIR_GPIO_A_START + i;
					PIO_InterruptDisable(AT91C_BASE_PIOA, i);
				}
					

				DEBUGMSG(DBG_LVL_IRQ,"%s(%d) IRQ_VIR_GPIO_A_Group pin = %d\n",__FUNCTION__,__LINE__,i);
			}
			else if(ActivatedBit = PIO_Interrupt(AT91C_BASE_PIOB, AT91C_ID_PIOA_B))
			{
				for(i=0;i<32;i++)
				{
					if((ActivatedBit >> i) == 0x01)
						break;
				}
				if(i < 32)
				{
					nHalIrq = IRQ_VIR_GPIO_B_START + i;
					PIO_InterruptDisable(AT91C_BASE_PIOB, i);
				}

				DEBUGMSG(DBG_LVL_IRQ,"%s(%d) IRQ_VIR_GPIO_B_Group pin = %d\n",__FUNCTION__,__LINE__,i);
			}
			else if(ActivatedBit = PIO_Interrupt(AT91C_BASE_PIOC, AT91C_ID_PIOC_D))
			{
				for(i=0;i<32;i++)
				{
					if((ActivatedBit >> i) == 0x01)
						break;
				}
				if(i < 32)
				{
					nHalIrq = IRQ_VIR_GPIO_C_START + i;
					PIO_InterruptDisable(AT91C_BASE_PIOC, i);
				}
					

				DEBUGMSG(DBG_LVL_IRQ,"%s(%d) IRQ_VIR_GPIO_C_Group pin = %d\n",__FUNCTION__,__LINE__,i);
			}
			else if(ActivatedBit = PIO_Interrupt(AT91C_BASE_PIOC, AT91C_ID_PIOC_D))
			{
				for(i=0;i<32;i++)
				{
					if((ActivatedBit >> i) == 0x01)
						break;
				}
				if(i < 32)
				{
					nHalIrq = IRQ_VIR_GPIO_D_START + i;
					PIO_InterruptDisable(AT91C_BASE_PIOD, i);
				}

				DEBUGMSG(DBG_LVL_IRQ,"%s(%d) IRQ_VIR_GPIO_D_Group pin = %d\n",__FUNCTION__,__LINE__,i);
			}
			else
			{
				DEBUGMSG(DBG_LVL_IRQ,"%s(%d) NOT INTERRUPT GPIO\n",__FUNCTION__,__LINE__);
			}

			pAIC->AIC_IDCR = (1 << nPhyIrq ); //disable AIC interrupt for this controller
		}
		else if(nHalIrq >= IRQ_VIR_DMA0_START && nHalIrq < IRQ_VIR_DMA0_END)//HDMA0 IRQ
		{
			int DMACh = (nHalIrq - IRQ_VIR_DMA0_START);
			//pHDMAReg = (AT91PS_HDMA)AT91C_BASE_HDMA;
			
			nPhyIrq = AT91C_ID_HDMA;
			pAIC->AIC_IDCR = (1 << nPhyIrq );
		}
		else if(nHalIrq >= IRQ_VIR_DMA1_START && nHalIrq < IRQ_VIR_DMA1_END)//HDMA1 IRQ
		{
			int DMACh = (nHalIrq - IRQ_VIR_DMA1_START);
			//pHDMAReg = (AT91PS_HDMA)AT91C_BASE_HDMA1;
			
			//nPhyIrq = AT91C_ID_HDMA1;
			pAIC->AIC_IDCR = (1 << nPhyIrq );
		}
        else
		{
    		nHalIrq = nPhyIrq;

			pAIC->AIC_IDCR = (1 << nPhyIrq ); //disable AIC interrupt for this controller
		}

		if (nHalIrq > OAL_INTR_IRQ_MAXIMUM)
			nSysIntr = SYSINTR_NOP;

        // We don't assume IRQ sharing, use STATIC mapping
        nSysIntr = OALIntrTranslateIrq(nHalIrq);

		// Unmask interrupts in case it's NOP or invalid. It is not called.
		if (SYSINTR_NOP == nSysIntr)
		{
			pAIC->AIC_IDCR = (1 << (nPhyIrq&0x1F) );
			DEBUGMSG(1,"%s(%d) SYSINTR_NOP\n",__FUNCTION__,__LINE__);
		}
	}

//DEBUGMSG(DBG_LVL_IRQ,"%s : nSysIntr = %d\n", __FUNCTION__, nSysIntr );
    return nSysIntr;
}


//------------------------------------------------------------------------------




/*
  Name: HAL_InterruptEnable()
  Copyright: 
  Author: jung byoung oh
  Date: 10-12-03 15:08
  Description: Enable hardware specific interrupt nubmer
        Input: virtual interrupt number.
*/
PUBLIC VOID
HAL_InterruptEnable( UINT32 nSysIntr )
{
    DEBUGMSG(DBG_LVL_IRQ_EN, "%s (%d): (%d)\n", __FUNCTION__, __LINE__, nSysIntr);

    // SYSINTR_VMINI & SYSINTR_TIMING are special cases
    //if ((nSysIntr == SYSINTR_VMINI) || (nSysIntr == SYSINTR_TIMING))
    if( /*(nSysIntr == SYSINTR_TICK) ||*/ (nSysIntr == SYSINTR_NOP) )
    {
        goto cleanUp;
    }

    if( nSysIntr == SYSINTR_TICK)
    {
        // enable interrupt mask
        AT91PS_AIC pAIC = (AT91PS_AIC)AT91C_BASE_AIC;
		pAIC->AIC_IECR = (1 << AT91C_ID_SYS ); //eanble AIC interrupt for this controller
        return;
    }

    // Enable interrupts
    OALIntrEnableIrqs( nSysIntr );

cleanUp:

    DEBUGMSG(DBG_LVL_IRQ_EN,"%s (%d): \n", __FUNCTION__, __LINE__ );

    return;
}

/*
  Name: HAL_InterruptDisable()
  Copyright: 
  Author: jung byoung oh
  Date: 10-12-03 15:10
  Description: disable the specific interrupt number
        Input: virtual interrupt number
*/
PUBLIC VOID
HAL_InterruptDisable( UINT32 nSysIntr )
{
    DEBUGMSG(DBG_LVL_IRQ, "%s : (%d)\n", __FUNCTION__, nSysIntr);

    if( (nSysIntr == SYSINTR_TICK) || (nSysIntr == SYSINTR_NOP) )
        return;
    
    // Disable interrupts
    OALIntrDisableIrqs( nSysIntr );

    DEBUGMSG(DBG_LVL_IRQ,"%s : \n", __FUNCTION__ );

    return;
}

/*
  Name: HAL_InterruptDone()
  Copyright: 
  Author: jung byoung oh
  Date: 10-12-03 15:11
  Description: do the end of interrupt action
        Input: virtual interrupt number
*/

PUBLIC VOID
HAL_InterruptDone( UINT32 nSysIntr )
{
    DEBUGMSG(DBG_LVL_IRQ, "%s (%d): (%d)\n", __FUNCTION__, __LINE__, nSysIntr);

    if( (nSysIntr == SYSINTR_TICK) || (nSysIntr == SYSINTR_NOP) )
        return;

    // Re-enable interrupts
    OALIntrDoneIrqs( nSysIntr );

    DEBUGMSG(DBG_LVL_IRQ,"%s (%d): Done\n", __FUNCTION__, __LINE__ );

    return;
}

